#########################################################################
#File Description: 
#CHANGE POINT DETECTION FOR SPEED AND DIRECTION
#########################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(circular)
library(changepoint)
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter4/funcs/CPDD.R")

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 30 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")


#***********************************************************************
#************CHANGE DETECTION FOR SPEED & DIRECTION DATA****************
#***********************************************************************
dir_to_test_change=dirs[,3] #mast that will be used for change detection
change_dir_list=CPDD(dir_to_test_change)

#Outcomes of change point detection for wind direction
changepoints = sort(unlist(change_dir_list))
changepoints = changepoints[!changepoints == 0]
maxids = max(changepoints)

#Change detection for the wind speed
#finding spatial averages of wind speed data to do a time series change test
aa = 1;speed.means = vector(length = Period)
for (i in 1:length(speed.means)){
  speed.means[i] = mean(datas$Speed[aa:(aa+199)])
  aa = aa + 200
}
#Change point detection for wind speed data
maxnumcps = length(speed.means)/3 #this is extreme case: assuming a change will happen every 3 hrs
ansmean = cpt.mean(speed.means,method="BinSeg",Q=maxnumcps)
changepoints_speed = ansmean@cpts #changepoints of speed
changepoints_speed = changepoints_speed + 1 #because the command identifies the point before change
changepoints_speed=changepoints_speed[-length(changepoints_speed)] #algorithm always identifies the last point as CP

#***********************************************************************
#**************************FINAL PLOTS**********************************
#***********************************************************************

par(mfrow=c(2,1),mar=c(5,6,2,2))
plot(seq(1,length(speed.means),1),speed.means,col="black",ylim=c(0,17),type="l",xlab="Time (hours)",ylab="Wind Speed (m/s)",cex=1.5,cex.axis=1.25,cex.lab=1.75,lwd=2)
box(lwd=2)
for (j in 1:length(changepoints_speed)){
  points(changepoints_speed[j],speed.means[changepoints_speed[j]],col="red",pch=4,cex=1.5,lwd=3)
  abline(v = changepoints_speed[j],lty=6,col="red",lwd=2)
}

plot(seq(1,length(dir_to_test_change),1),dir_to_test_change,col="black",pch=3,ylim=c(0,360),type="p",xlab="Time (hours)",ylab="Wind direction (degs)",cex=1.5,cex.axis=1.25,cex.lab=1.75,lwd=1)
box(lwd=2)
for (j in 1:length(changepoints)){
  points(changepoints[j],dir_to_test_change[changepoints[j]],col="blue",pch=4,cex=1.75,lwd=3)
  abline(v = changepoints[j],lty=6,col="blue",lwd=2)
}
