#########################################################################
#File Description: 
#This file generates Fig 4.9 and Tables 4.5, 4.6
#########################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter4/funcs/bin_est.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
roll.int=6# amount by which we translate every roll
r_speeds = c(0.00, 4.50, 9.00) #refined speed thresholds
r_dirs = c(45,225) #refined direction thresholds
num_regimes = 6 #number of regimes
infpt = 9.5 #inflection point in power curve
vco = 20 #cutout speed 
vr = 13.5 #rated speed
vci = 3.5 #cutin speed
num_regimes = 6 #number of regimes

#################################################################################
####################  IMPORTING DATA  ###########################################
#################################################################################
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

load("Code Used in the Book/Chapter4/temp/trainout_test.R")
load("Code Used in the Book/Chapter4/temp/predout_test.R")
load("Code Used in the Book/Chapter4/temp/indicesout_test.R")

########################################################################################
############TURBINE-SPECIFIC POWER ESTIMATION USING BINNING METHOD######################
########################################################################################
output.list = bin_est(datas,bw=.5,cutin = vci,rated = vr,numturbs = N)
vbin = as.numeric(output.list[[1]])
P.est = output.list[[2]]

##########################################################################
########### COMPARING FORECASTS FROM DIFFERENT MODELS    #################
##########################################################################
num_rolls = length(pred.out) #number of forecasting rolls
num_test = 12 #forecasting horizon
num_turbs = 200 #number of turbines

p_asym = p_sep = p_per = truth    = v_asym = v_sep = betas  = vector(length = num_turbs*num_rolls*num_test)
asym.perc = matrix(nrow=num_turbs*num_rolls*num_test,ncol=6)
truth.p = 0;z = 1
adj_mat = matrix(nrow=num_turbs*num_rolls*num_test,ncol=2)
adj_mat_perc = matrix(nrow=num_turbs*num_rolls*num_test,ncol=6)

for (i in 1:num_rolls){
  #Recording forecasts
  p_asym[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,1])
  p_sep[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,4])
  p_per[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,2])
  truth[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,3])
  v_asym[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,5])
  v_sep[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,6])
  betas[z:(z+num_test*num_turbs-1)] = as.numeric(pred.out[[i]][,7])

  asym.perc[z:(z+num_test*num_turbs-1),1] = qnorm(p=0.4,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  asym.perc[z:(z+num_test*num_turbs-1),2] = qnorm(p=0.5,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  asym.perc[z:(z+num_test*num_turbs-1),3] = qnorm(p=0.6,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  asym.perc[z:(z+num_test*num_turbs-1),4] = qnorm(p=0.7,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  asym.perc[z:(z+num_test*num_turbs-1),5] = qnorm(p=0.73,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  asym.perc[z:(z+num_test*num_turbs-1),6] = qnorm(p=0.8,mean = p_asym[z:(z+num_test*num_turbs-1)],sd=sqrt(v_asym[z:(z+num_test*num_turbs-1)]))
  
  
  for(j in 1:dim(adj_mat)[2]){
    if(j == 1){jj = 8};if(j == 2){jj=10} #unconstrained/constrained calibration
     pred.out[[i]][1:400,jj] = pred.out[[i]][1:400,1] #first 2 hours - no calibration
     adj_mat[z:(z+num_test*num_turbs-1),j] = as.numeric(pred.out[[i]][,jj]) 
  }
 
  cquant = pmin(3, betas[z:(z+num_test*num_turbs-1)])
  cquant = pmax(-3,cquant)
  adj_mat_perc[z:(z+num_test*num_turbs-1),1] = asym.perc[z:(z+num_test*num_turbs-1),1] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),1] = asym.perc[z:(z+2*num_turbs-1),1]
  adj_mat_perc[z:(z+num_test*num_turbs-1),2] = asym.perc[z:(z+num_test*num_turbs-1),2] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),2] = asym.perc[z:(z+2*num_turbs-1),2]
  adj_mat_perc[z:(z+num_test*num_turbs-1),3] = asym.perc[z:(z+num_test*num_turbs-1),3] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),3] = asym.perc[z:(z+2*num_turbs-1),3]
  adj_mat_perc[z:(z+num_test*num_turbs-1),4] = asym.perc[z:(z+num_test*num_turbs-1),4] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),4] = asym.perc[z:(z+2*num_turbs-1),4]
  adj_mat_perc[z:(z+num_test*num_turbs-1),5] = asym.perc[z:(z+num_test*num_turbs-1),5] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),5] = asym.perc[z:(z+2*num_turbs-1),5]
  adj_mat_perc[z:(z+num_test*num_turbs-1),6] = asym.perc[z:(z+num_test*num_turbs-1),6] + cquant
  adj_mat_perc[z:(z+2*num_turbs-1),6] = asym.perc[z:(z+2*num_turbs-1),6]
  
  
  z = z + num_test*num_turbs
  st.ts = indices.out[[i]][2]+1
  end.ts = st.ts + num_test - 1
  sel.p = datas$Power[ (((st.ts-1)*num_turbs)+1): (end.ts*num_turbs)]
  truth.p = c(truth.p,sel.p)
  if(i %in% seq(1,num_rolls,100)){cat(paste(i,"out of",num_rolls,sep=" "),"\n")}
}
truth.p = truth.p[-1]

#######Per-hour forecast
num_test = 12
a_h = p_h = s_h = t_h = c1_h = vc1_h = vs_h = matrix(nrow=num_rolls*N,ncol=num_test)
aa = 1
for (i in 1:num_rolls){
  a_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,1])
  a_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,1])
  a_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,1])
  a_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,1])
  a_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,1])
  a_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,1])
  a_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,1])
  a_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,1])
  a_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,1])
  a_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,1])
  a_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,1])
  a_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,1])
  
  p_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,2])
  p_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,2])
  p_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,2])
  p_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,2])
  p_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,2])
  p_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,2])
  p_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,2])
  p_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,2])
  p_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,2])
  p_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,2])
  p_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,2])
  p_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,2])
  
  s_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,4])
  s_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,4])
  s_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,4])
  s_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,4])
  s_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,4])
  s_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,4])
  s_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,4])
  s_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,4])
  s_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,4])
  s_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,4])
  s_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,4])
  s_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,4])
  
  t_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,3])
  t_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,3])
  t_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,3])
  t_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,3])
  t_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,3])
  t_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,3])
  t_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,3])
  t_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,3])
  t_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,3])
  t_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,3])
  t_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,3])
  t_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,3])
  
  c1_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,10])
  c1_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,10])
  c1_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,10])
  c1_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,10])
  c1_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,10])
  c1_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,10])
  c1_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,10])
  c1_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,10])
  c1_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,10])
  c1_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,10])
  c1_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,10])
  c1_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,10])
  
  vc1_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,5])
  vc1_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,5])
  vc1_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,5])
  vc1_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,5])
  vc1_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,5])
  vc1_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,5])
  vc1_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,5])
  vc1_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,5])
  vc1_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,5])
  vc1_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,5])
  vc1_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,5])
  vc1_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,5])
  
  vs_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,6])
  vs_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,6])
  vs_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,6])
  vs_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,6])
  vs_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,6])
  vs_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,6])
  vs_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,6])
  vs_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,6])
  vs_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,6])
  vs_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,6])
  vs_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,6])
  vs_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,6])
  aa = aa + 200
}

err_a = err_p = err_s = err_c1  = list(length = num_test)
for (h in 1:num_test){
  err_a[[h]] = (a_h[,h] - t_h[,h])^2
  err_s[[h]] = (s_h[,h] - t_h[,h])^2
  err_p[[h]] = (p_h[,h] - t_h[,h])^2
  err_c1[[h]] = (c1_h[,h] - t_h[,h])^2
}

MAE_h = matrix(nrow=10,ncol=num_test)
for(i in 1:num_test){
  MAE_h[1,i] = mean(sqrt(err_a[[i]]))
  MAE_h[2,i] = mean(sqrt(err_s[[i]]))
  MAE_h[4,i] = mean(sqrt(err_p[[i]]))
  MAE_h[5,i] = mean(sqrt(err_c1[[i]]))
}
rownames(MAE_h)=c("ASYM","SEP1","SEP2","PER","Adj1","RSAR","MSAR-s","MSAR-h","MSVAR-s","MSVAR-h")
MAE_h

##Power forecasts
test.P = truth.p
c1_bound = adj_mat[,2]
c1_bound_perc = adj_mat_perc

Pestimations = list(length = N)
for (ii in 1:N){
  ind = seq(ii,(length(p_asym)-200)+ii,N)
  pred_a = p_asym[ind]
  pred_s = p_sep[ind]
  pred_p = p_per[ind]
  pred_c1 = c1_bound[ind]

  pred_q1 = c1_bound_perc[ind,1]
  pred_q2 = c1_bound_perc[ind,2]
  pred_q3 = c1_bound_perc[ind,3]
  pred_q4 = c1_bound_perc[ind,4]
  pred_q5 = c1_bound_perc[ind,5]
  pred_q6 = c1_bound_perc[ind,6]
  
  preda_q1 = asym.perc[ind,1]
  preda_q2 = asym.perc[ind,2]
  preda_q3 = asym.perc[ind,3]
  preda_q4 = asym.perc[ind,4]
  preda_q5 = asym.perc[ind,5]
  preda_q6 = asym.perc[ind,6]
  
  
  tr = truth[ind]
  powerpred_a = powerpred_s = powerpred_p = powerpred_c1  = vector(length = length(pred_a))
  powerpred_q1  = powerpred_q2  =powerpred_q3  =powerpred_q4  =powerpred_q5  =powerpred_q6  =vector(length = length(pred_a))
  powerpreda_q1  = powerpreda_q2  =powerpreda_q3  =powerpreda_q4  =powerpreda_q5  =powerpreda_q6  =vector(length = length(pred_a))
  
  powercurve.est = P.est[[ii]]
  powerpred_a[which(pred_a < vbin[1])] = 0
  powerpred_s[which(pred_s < vbin[1])] = 0
  powerpred_p[which(pred_p < vbin[1])] = 0
  powerpred_c1[which(pred_c1 < vbin[1])] = 0
  powerpred_q1[which(pred_q1 < vbin[1])] = 0
  powerpred_q2[which(pred_q2 < vbin[1])] = 0
  powerpred_q3[which(pred_q3 < vbin[1])] = 0
  powerpred_q4[which(pred_q4 < vbin[1])] = 0
  powerpred_q5[which(pred_q5 < vbin[1])] = 0
  powerpred_q6[which(pred_q6 < vbin[1])] = 0
  powerpreda_q1[which(preda_q1 < vbin[1])] = 0
  powerpreda_q2[which(preda_q2 < vbin[1])] = 0
  powerpreda_q3[which(preda_q3 < vbin[1])] = 0
  powerpreda_q4[which(preda_q4 < vbin[1])] = 0
  powerpreda_q5[which(preda_q5 < vbin[1])] = 0
  powerpreda_q6[which(preda_q6 < vbin[1])] = 0
  
  powerpred_a[which(pred_a > vr)] = max(powercurve.est)
  powerpred_s[which(pred_s > vr)] = max(powercurve.est)
  powerpred_p[which(pred_p > vr)] = max(powercurve.est)
  powerpred_c1[which(pred_c1 > vr)] = max(powercurve.est)
  powerpred_q1[which(pred_q1 > vr)] = max(powercurve.est)
  powerpred_q2[which(pred_q2 > vr)] = max(powercurve.est)
  powerpred_q3[which(pred_q3 > vr)] = max(powercurve.est)
  powerpred_q4[which(pred_q4 > vr)] = max(powercurve.est)
  powerpred_q5[which(pred_q5 > vr)] = max(powercurve.est)
  powerpred_q6[which(pred_q6 > vr)] = max(powercurve.est)
  powerpreda_q1[which(preda_q1 > vr)] = max(powercurve.est)
  powerpreda_q2[which(preda_q2 > vr)] = max(powercurve.est)
  powerpreda_q3[which(preda_q3 > vr)] = max(powercurve.est)
  powerpreda_q4[which(preda_q4 > vr)] = max(powercurve.est)
  powerpreda_q5[which(preda_q5 > vr)] = max(powercurve.est)
  powerpreda_q6[which(preda_q6 > vr)] = max(powercurve.est)
  
  #Based on Binning method
  for (i in 1:(length(vbin)-1)){
    powerpred_a[which(pred_a >= vbin[i] & pred_a < vbin[i+1])] = powercurve.est[i]
    powerpred_s[which(pred_s >= vbin[i] & pred_s < vbin[i+1])] = powercurve.est[i]
    powerpred_p[which(pred_p >= vbin[i] & pred_p < vbin[i+1])] = powercurve.est[i]
    powerpred_c1[which(pred_c1 >= vbin[i] & pred_c1 < vbin[i+1])] = powercurve.est[i]
    powerpred_q1[which(pred_q1 >= vbin[i] & pred_q1 < vbin[i+1])] = powercurve.est[i]
    powerpred_q2[which(pred_q2 >= vbin[i] & pred_q2 < vbin[i+1])] = powercurve.est[i]
    powerpred_q3[which(pred_q3 >= vbin[i] & pred_q3 < vbin[i+1])] = powercurve.est[i]
    powerpred_q4[which(pred_q4 >= vbin[i] & pred_q4 < vbin[i+1])] = powercurve.est[i]
    powerpred_q5[which(pred_q5 >= vbin[i] & pred_q5 < vbin[i+1])] = powercurve.est[i]
    powerpred_q6[which(pred_q6 >= vbin[i] & pred_q6 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q1[which(preda_q1 >= vbin[i] & preda_q1 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q2[which(preda_q2 >= vbin[i] & preda_q2 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q3[which(preda_q3 >= vbin[i] & preda_q3 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q4[which(preda_q4 >= vbin[i] & preda_q4 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q5[which(preda_q5 >= vbin[i] & preda_q5 < vbin[i+1])] = powercurve.est[i]
    powerpreda_q6[which(preda_q6 >= vbin[i] & preda_q6 < vbin[i+1])] = powercurve.est[i]
    
    
    }
  
  truetest = test.P[ind]
  
  mat.out = matrix(ncol=17,nrow=length(truetest))
  mat.out[,1] = powerpred_a
  mat.out[,2] = powerpred_s
  mat.out[,3] = powerpred_p
  mat.out[,4] = truetest
  mat.out[,5] = powerpred_c1
  mat.out[,6] = powerpred_q1
  mat.out[,7] = powerpred_q2
  mat.out[,8] = powerpred_q3
  mat.out[,9] = powerpred_q4
  mat.out[,10] = powerpred_q5
  mat.out[,11] = powerpred_q6
  mat.out[,12] = powerpreda_q1
  mat.out[,13] = powerpreda_q2
  mat.out[,14] = powerpreda_q3
  mat.out[,15] = powerpreda_q4
  mat.out[,16] = powerpreda_q5
  mat.out[,17] = powerpreda_q6
  Pestimations[[ii]] = mat.out
  if (ii %in% seq(1,num_turbs,20)){cat(ii,"  ")}
}

SE_a = SE_s = SE_p = SE_c1  = 0
for(i in 1:200){
  truetest = Pestimations[[i]][,4]
  
  #Removing outliers from power values
  out1 = which(truetest < 0)
  out2 = which(is.na(truetest) == T)
  out3 = unique(c(out1,out2))
  
  tempp1 = (Pestimations[[i]][-out3,1] - Pestimations[[i]][-out3,4])^2
  tempp2 = (Pestimations[[i]][-out3,2] - Pestimations[[i]][-out3,4])^2
  tempp3 = (Pestimations[[i]][-out3,3] - Pestimations[[i]][-out3,4])^2
  tempp4 = (Pestimations[[i]][-out3,5] - Pestimations[[i]][-out3,4])^2
  
  SE_a = c(SE_a,tempp1)
  SE_s = c(SE_s,tempp2)
  SE_p = c(SE_p,tempp3)
  SE_c1 = c(SE_c1,tempp4)
  
}
SE_a = SE_a[-1];SE_s = SE_s[-1];SE_p = SE_p[-1];SE_c1 = SE_c1[-1]

#Prediction for each horizon
num_test = 12
pest_h_a = pest_h_s = pest_h_p = ptrue_h = pest_h_c1  = list(length = num_test)
for (hh in 1:num_test){
  temp1 = temp2 = temp3 = temp4 = temp5  = 0
  inds = seq(hh,(num_test*num_rolls),num_test)
  for (i in 1:200){
    temp1 = c(temp1,Pestimations[[i]][inds,1])
    temp2 = c(temp2,Pestimations[[i]][inds,2])
    temp3 = c(temp3,Pestimations[[i]][inds,3])
    temp4 = c(temp4,Pestimations[[i]][inds,4])
    temp5 = c(temp5,Pestimations[[i]][inds,5])
  }
  temp1 = temp1[-1];temp2=temp2[-1];temp3=temp3[-1];temp4=temp4[-1];temp5=temp5[-1]
  
  pest_h_a[[hh]] = temp1
  pest_h_s[[hh]] = temp2
  pest_h_p[[hh]] = temp3
  ptrue_h[[hh]] = temp4
  pest_h_c1[[hh]] = temp5
  
  out1 = which(temp4 < 0)
  out2 = which(is.na(temp4) == T)
  out = unique(c(out1,out2))
  
  cat(paste(hh,"out of",num_test,sep=" "),"\n")
}

errpower_a = errpower_p = errpower_s = errpower_c1  = list(length = num_test)
for (h in 1:num_test){
  errpower_a[[h]] = (pest_h_a[[h]] - ptrue_h[[h]])^2
  errpower_s[[h]] = (pest_h_s[[h]] - ptrue_h[[h]])^2
  errpower_p[[h]] = (pest_h_p[[h]] - ptrue_h[[h]])^2
  errpower_c1[[h]] = (pest_h_c1[[h]] - ptrue_h[[h]])^2
}

MAE_hp = matrix(nrow=10,ncol=num_test)
for(i in 1:num_test){
  out1 = which(ptrue_h[[i]] < 0)
  out2 = which(is.na(ptrue_h[[i]]) == T)
  out = unique(c(out1,out2))
  MAE_hp[1,i] = mean(sqrt(errpower_a[[i]][-out]),na.rm=T)
  MAE_hp[2,i] = mean(sqrt(errpower_s[[i]][-out]),na.rm=T)
  MAE_hp[4,i] = mean(sqrt(errpower_p[[i]][-out]),na.rm=T)
  MAE_hp[5,i] = mean(sqrt(errpower_c1[[i]][-out]),na.rm=T)
}
rownames(MAE_hp)=c("ASYM","SEP1","SEP2","PER","Adj1","RSAR","MSAR-s","MSAR-h","MSVAR-s","MSVAR-h")
MAE_hp
Pestimations_all = Pestimations


###################SEP results###################
load("Code Used in the Book/Chapter4/temp/SEP/predout_test.R")
load("Code Used in the Book/Chapter4/temp/SEP/indicesout_test.R")

#Computing 12-hr average for wind speed
num_test = 12
p_sep2  = truth = truth.p = v_sep2 = 0
sep.perc = matrix(nrow=num_rolls*num_test*num_turbs,ncol=6)
z = 1
for (i in 1:num_rolls){
  p_sep2 = c(p_sep2,as.numeric(pred.out[[i]][,1]))
  truth = c(truth,as.numeric(pred.out[[i]][,3]))
  v_sep2 = c(v_sep2,as.numeric(pred.out[[i]][,5]))
  
  if(i == 1){p_sep2 = p_sep2[-1];truth=truth[-1];v_sep2 = v_sep2[-1]}
  
  sep.perc[z:(z+num_test*num_turbs-1),1] = qnorm(p=0.4,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  sep.perc[z:(z+num_test*num_turbs-1),2] = qnorm(p=0.5,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  sep.perc[z:(z+num_test*num_turbs-1),3] = qnorm(p=0.6,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  sep.perc[z:(z+num_test*num_turbs-1),4] = qnorm(p=0.7,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  sep.perc[z:(z+num_test*num_turbs-1),5] = qnorm(p=0.73,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  sep.perc[z:(z+num_test*num_turbs-1),6] = qnorm(p=0.8,mean = p_sep2[z:(z+num_test*num_turbs-1)],sd=sqrt(v_sep2[z:(z+num_test*num_turbs-1)]))
  
  
  st.ts = indices.out[[i]][2]+1
  end.ts = st.ts + num_test - 1
  sel.p = datas$Power[ (((st.ts-1)*200)+1): (end.ts*200)]
  truth.p = c(truth.p,sel.p)
  z = z + num_test*num_turbs
}
truth.p = truth.p[-1]

##per horizon
num_test = 12
s2_h = t_h = vs2_h = matrix(nrow=num_rolls*200,ncol=num_test)
aa = 1
for (i in 1:num_rolls){
  
  s2_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,1])
  s2_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,1])
  s2_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,1])
  s2_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,1])
  s2_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,1])
  s2_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,1])
  s2_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,1])
  s2_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,1])
  s2_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,1])
  s2_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,1])
  s2_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,1])
  s2_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,1])
  
  t_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,3])
  t_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,3])
  t_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,3])
  t_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,3])
  t_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,3])
  t_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,3])
  t_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,3])
  t_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,3])
  t_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,3])
  t_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,3])
  t_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,3])
  t_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,3])
  
  vs2_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,5])
  vs2_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,5])
  vs2_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,5])
  vs2_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,5])
  vs2_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,5])
  vs2_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,5])
  vs2_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,5])
  vs2_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,5])
  vs2_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,5])
  vs2_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,5])
  vs2_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,5])
  vs2_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,5])
  
  aa = aa + 200
}

err_s2 = list(length = num_test)
for (h in 1:num_test){
  err_s2[[h]] = (s2_h[,h] - t_h[,h])^2
  
}
for(i in 1:num_test){
  MAE_h[3,i] = mean(sqrt(err_s2[[i]]))
}
MAE_h

#####SEP2: projecting to power domain
num_test = 12
test.P = truth.p
Pestimations = list(length = 200)
for (ii in 1:200){
  ind = seq(ii,(length(p_sep2)-200)+ii,200)
  pred_s2 = p_sep2[ind]
  pred_q1 = sep.perc[ind,1]
  pred_q2 = sep.perc[ind,2]
  pred_q3 = sep.perc[ind,3]
  pred_q4 = sep.perc[ind,4]
  pred_q5 = sep.perc[ind,5]
  pred_q6 = sep.perc[ind,6]
  
  
  
  tr = truth[ind]
  powerpred_s2 = vector(length = length(pred_s2))
  powerpred_q1 = powerpred_q2 = powerpred_q3 = powerpred_q4 = powerpred_q5 = powerpred_q6 = vector(length = length(pred_s2))
  powercurve.est = P.est[[ii]]
  powerpred_s2[which(pred_s2 < vbin[1])] = 0
  powerpred_s2[which(pred_s2 > vr)] = max(powercurve.est)
  
  powerpred_q1[which(pred_q1 < vbin[1])] = 0
  powerpred_q1[which(pred_q1 > vr)] = max(powercurve.est)
  powerpred_q2[which(pred_q2 < vbin[1])] = 0
  powerpred_q2[which(pred_q2 > vr)] = max(powercurve.est)
  powerpred_q3[which(pred_q3 < vbin[1])] = 0
  powerpred_q3[which(pred_q3 > vr)] = max(powercurve.est)
  powerpred_q4[which(pred_q4 < vbin[1])] = 0
  powerpred_q4[which(pred_q4 > vr)] = max(powercurve.est)
  powerpred_q5[which(pred_q5 < vbin[1])] = 0
  powerpred_q5[which(pred_q5 > vr)] = max(powercurve.est)
  powerpred_q6[which(pred_q6 < vbin[1])] = 0
  powerpred_q6[which(pred_q6 > vr)] = max(powercurve.est)
  
  #Based on Binning method
  for (i in 1:(length(vbin)-1)){
    powerpred_s2[which(pred_s2 >= vbin[i] & pred_s2 < vbin[i+1])] = powercurve.est[i]
    powerpred_q1[which(pred_q1 >= vbin[i] & pred_q1 < vbin[i+1])] = powercurve.est[i]
    powerpred_q2[which(pred_q2 >= vbin[i] & pred_q2 < vbin[i+1])] = powercurve.est[i]
    powerpred_q3[which(pred_q3 >= vbin[i] & pred_q3 < vbin[i+1])] = powercurve.est[i]
    powerpred_q4[which(pred_q4 >= vbin[i] & pred_q4 < vbin[i+1])] = powercurve.est[i]
    powerpred_q5[which(pred_q5 >= vbin[i] & pred_q5 < vbin[i+1])] = powercurve.est[i]
    powerpred_q6[which(pred_q6 >= vbin[i] & pred_q6 < vbin[i+1])] = powercurve.est[i]
    
    }
  
  truetest = test.P[ind]
  
  mat.out = matrix(ncol=8,nrow=length(truetest))
  mat.out[,1] = powerpred_s2
  mat.out[,2] = truetest
  mat.out[,3] = powerpred_q1
  mat.out[,4] = powerpred_q2
  mat.out[,5] = powerpred_q3
  mat.out[,6] = powerpred_q4
  mat.out[,7] = powerpred_q5
  mat.out[,8] = powerpred_q6
  Pestimations[[ii]] = mat.out
  if (ii %in% seq(1,200,20)){cat(ii,"  ")}
}

SE_s2 =  0
for(i in 1:200){
  truetest = Pestimations[[i]][,2]
  out1 = which(truetest < 0)
  out2 = which(is.na(truetest) == T)
  out3 = unique(c(out1,out2))
  
  tempp1 = (Pestimations[[i]][-out3,1] - Pestimations[[i]][-out3,2])^2
  
  SE_s2 = c(SE_s2,tempp1)
  
}
SE_s2 = SE_s2[-1]


#Prediction for each horizon
num_test = 12
pest_h_s2 = list(length = num_test)
for (hh in 1:num_test){
  temp1 =  temp2 = 0
  inds = seq(hh,(num_test*num_rolls),num_test)
  for (i in 1:200){
    temp1 = c(temp1,Pestimations[[i]][inds,1])
    temp2 = c(temp2,Pestimations[[i]][inds,2])
  }
  temp1 = temp1[-1];temp2=temp2[-1]
  pest_h_s2[[hh]] = temp1
  ptrue_h[[hh]] = temp2
  
  cat(hh,"\n")
}

errpower_s2 = list(length = num_test)
for (h in 1:num_test){
  errpower_s2[[h]] = (pest_h_s2[[h]] - ptrue_h[[h]])^2
}

for(i in 1:num_test){
  out1 = which(ptrue_h[[i]] < 0)
  out2 = which(is.na(ptrue_h[[i]]) == T)
  out = unique(c(out1,out2))
  MAE_hp[3,i] = mean(sqrt(errpower_s2[[i]][-out]),na.rm=T)
}
MAE_hp

Pestimations_sep2 = Pestimations

###################RSAR###################
setwd("Code Used in the Book/Chapter4/temp/RSAR")
load("predout_test.R")
load("indicesout_test.R")
load("paramsout_test.R")

#Computing 12-hr average for wind speed
num_test = 12
p_rs1 = p_rs2  = truth = truth.p = 0
rsar.perc = matrix(nrow=num_test*num_rolls*N,ncol=6)
zz = 1
for (i in 1:num_rolls){
  p_rs1 = c(p_rs1,as.numeric(pred.out[[i]][,1]))
  truth = c(truth,as.numeric(pred.out[[i]][,3]))
  
  #quantile data
  if(i == 1){p_rs1 = p_rs1[-1];truth = truth[-1]}
  
  mtt = matrix(nrow=2400,ncol=6)
  for(turb in 1:N){
  est_variance = params.out[[i]][turb,3] #variance for the turbine
  est_mean = p_rs1[zz:(zz+2399)] #vector of means for every horizon
  est_mean = est_mean[seq(turb,2400,200)]
  mtt[seq(turb,2400,200),1] = qnorm(p=0.4,mean=est_mean,sd=sqrt(est_variance))
  mtt[seq(turb,2400,200),2] = qnorm(p=0.5,mean=est_mean,sd=sqrt(est_variance))
  mtt[seq(turb,2400,200),3] = qnorm(p=0.6,mean=est_mean,sd=sqrt(est_variance))
  mtt[seq(turb,2400,200),4] = qnorm(p=0.7,mean=est_mean,sd=sqrt(est_variance))
  mtt[seq(turb,2400,200),5] = qnorm(p=0.73,mean=est_mean,sd=sqrt(est_variance))
  mtt[seq(turb,2400,200),6] = qnorm(p=0.8,mean=est_mean,sd=sqrt(est_variance))
  }
  rsar.perc[zz:(zz+2399),1:6] = mtt
  zz =zz + 2400
  st.ts = indices.out[[i]][2]+1
  end.ts = st.ts + num_test - 1
  sel.p = datas$Power[ (((st.ts-1)*200)+1): (end.ts*200)]
  truth.p = c(truth.p,sel.p)
}
truth.p = truth.p[-1]

#per horizon
num_test = 12
rs1_h   = t_h = matrix(nrow=num_rolls*200,ncol=num_test)
aa = 1
for (i in 1:num_rolls){
  rs1_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,1])
  rs1_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,1])
  rs1_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,1])
  rs1_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,1])
  rs1_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,1])
  rs1_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,1])
  rs1_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,1])
  rs1_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,1])
  rs1_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,1])
  rs1_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,1])
  rs1_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,1])
  rs1_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,1])
  
  t_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,3])
  t_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,3])
  t_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,3])
  t_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,3])
  t_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,3])
  t_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,3])
  t_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,3])
  t_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,3])
  t_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,3])
  t_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,3])
  t_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,3])
  t_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,3])
  
  
  aa = aa + 200
}

err_rs1 = err_rs2 = list(length = num_test)
for (h in 1:num_test){
  err_rs1[[h]] = (rs1_h[,h] - t_h[,h])^2
}
for(i in 1:num_test){
  MAE_h[6,i] = mean(sqrt(err_rs1[[i]]))
}
MAE_h

#####RSAR: projecting to power domain
num_test = 12
test.P = truth.p
Pestimations = list(length = 200)
for (ii in 1:200){
  ind = seq(ii,(length(p_rs1)-200)+ii,200)
  pred_rs1 = p_rs1[ind]
  
  pred_q1 = rsar.perc[ind,1]
  pred_q2 = rsar.perc[ind,2]
  pred_q3 = rsar.perc[ind,3]
  pred_q4 = rsar.perc[ind,4]
  pred_q5 = rsar.perc[ind,5]
  pred_q6 = rsar.perc[ind,6]
  
  tr = truth[ind]
  powerpred_rs1 = vector(length = length(pred_rs1))
  powerpred_q1 = powerpred_q2 = powerpred_q3 = powerpred_q4 = powerpred_q5 = powerpred_q6 = vector(length = length(pred_rs1))
  
  powercurve.est = P.est[[ii]]
  powerpred_rs1[which(pred_rs1 < vbin[1])] = 0
  powerpred_rs1[which(pred_rs1 > vr)] = max(powercurve.est)
  
  powerpred_q1[which(pred_q1 < vbin[1])] = 0
  powerpred_q1[which(pred_q1 > vr)] = max(powercurve.est)
  powerpred_q2[which(pred_q2 < vbin[1])] = 0
  powerpred_q2[which(pred_q2 > vr)] = max(powercurve.est)
  powerpred_q3[which(pred_q3 < vbin[1])] = 0
  powerpred_q3[which(pred_q3 > vr)] = max(powercurve.est)
  powerpred_q4[which(pred_q4 < vbin[1])] = 0
  powerpred_q4[which(pred_q4 > vr)] = max(powercurve.est)
  powerpred_q5[which(pred_q5 < vbin[1])] = 0
  powerpred_q5[which(pred_q5 > vr)] = max(powercurve.est)
  powerpred_q6[which(pred_q6 < vbin[1])] = 0
  powerpred_q6[which(pred_q6 > vr)] = max(powercurve.est)
  
  
  #Based on Binning method
  for (i in 1:(length(vbin)-1)){
    powerpred_rs1[which(pred_rs1 >= vbin[i] & pred_rs1 < vbin[i+1])] = powercurve.est[i]
    powerpred_q1[which(pred_q1 >= vbin[i] & pred_q1 < vbin[i+1])] = powercurve.est[i]
    powerpred_q2[which(pred_q2 >= vbin[i] & pred_q2 < vbin[i+1])] = powercurve.est[i]
    powerpred_q3[which(pred_q3 >= vbin[i] & pred_q3 < vbin[i+1])] = powercurve.est[i]
    powerpred_q4[which(pred_q4 >= vbin[i] & pred_q4 < vbin[i+1])] = powercurve.est[i]
    powerpred_q5[which(pred_q5 >= vbin[i] & pred_q5 < vbin[i+1])] = powercurve.est[i]
    powerpred_q6[which(pred_q6 >= vbin[i] & pred_q6 < vbin[i+1])] = powercurve.est[i]
    
    }
  
  truetest = test.P[ind]

  mat.out = matrix(ncol=8,nrow=length(truetest))
  mat.out[,1] = powerpred_rs1
  mat.out[,2] = truetest
  mat.out[,3] = powerpred_q1
  mat.out[,4] = powerpred_q2
  mat.out[,5] = powerpred_q3
  mat.out[,6] = powerpred_q4
  mat.out[,7] = powerpred_q5
  mat.out[,8] = powerpred_q6
  Pestimations[[ii]] = mat.out
  if (ii %in% seq(1,200,20)){cat(ii,"  ")}
}

SE_rs1 =  0
for(i in 1:200){
  truetest = Pestimations[[i]][,2]
  out1 = which(truetest < 0)
  out2 = which(is.na(truetest) == T)
  out3 = unique(c(out1,out2))
  
  tempp1 = (Pestimations[[i]][-out3,1] - Pestimations[[i]][-out3,2])^2
  
  SE_rs1 = c(SE_rs1,tempp1)
  
}
SE_rs1 = SE_rs1[-1]

#Prediction for each horizon
num_test = 12
pest_h_rs1 = list(length = num_test)
for (hh in 1:num_test){
  temp1 =  temp2 = 0
  inds = seq(hh,(num_test*num_rolls),num_test)
  for (i in 1:200){
    temp1 = c(temp1,Pestimations[[i]][inds,1])
    temp2 = c(temp2,Pestimations[[i]][inds,2])
  }
  temp1 = temp1[-1];temp2=temp2[-1]
  pest_h_rs1[[hh]] = temp1
  ptrue_h[[hh]] = temp2
  
  cat(hh,"\n")
}

errpower_rs1 = list(length = num_test)
for (h in 1:num_test){
  errpower_rs1[[h]] = (pest_h_rs1[[h]] - ptrue_h[[h]])^2
}

for(i in 1:num_test){
  out1 = which(ptrue_h[[i]] < 0)
  out2 = which(is.na(ptrue_h[[i]]) == T)
  out = unique(c(out1,out2))
  MAE_hp[6,i] = mean(sqrt(errpower_rs1[[i]][-out]),na.rm=T)
}
MAE_hp
Pestimations_rs = Pestimations

##########################################################################
########################## MSAR results  #################################
##########################################################################
setwd("H:/DingBook/Files Sharing/Code Used in the Book/Chapter4/temp/MSAR")
load("predout_test.R")
load("indicesout_test.R")
load("paramsout_test.R")
load("paramsout2_test.R")


#Computing 12-hr average for wind speed
num_test = 12
p_mrs1 = p_mrs2  = truth = truth.p = 0
#msar.perc = matrix(nrow=N*num_test*num_rolls,ncol=6)
#zz = 1
for (i in 1:num_rolls){
  p_mrs1 = c(p_mrs1,as.numeric(pred.out[[i]][,1]))
  p_mrs2 = c(p_mrs2,as.numeric(pred.out[[i]][,4]))
  truth = c(truth,as.numeric(pred.out[[i]][,3]))
  
  
  #quantile data
  if(i == 1){p_mrs1 = p_mrs1[-1];p_mrs2 = p_mrs2[-1];truth = truth[-1]}
  
  st.ts = indices.out[[i]][2]+1
  end.ts = st.ts + num_test - 1
  sel.p = datas$Power[ (((st.ts-1)*200)+1): (end.ts*200)]
  truth.p = c(truth.p,sel.p)
  if (i %in% seq(1,1400,10)){cat(i,"  ")}
}
truth.p = truth.p[-1]

#Per horizon
num_test = 12
mrs1_h = mrs2_h = t_h = matrix(nrow=num_rolls*200,ncol=num_test)
aa = 1
for (i in 1:num_rolls){
  mrs1_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,1])
  mrs1_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,1])
  mrs1_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,1])
  mrs1_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,1])
  mrs1_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,1])
  mrs1_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,1])
  mrs1_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,1])
  mrs1_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,1])
  mrs1_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,1])
  mrs1_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,1])
  mrs1_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,1])
  mrs1_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,1])
  
  mrs2_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,4])
  mrs2_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,4])
  mrs2_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,4])
  mrs2_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,4])
  mrs2_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,4])
  mrs2_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,4])
  mrs2_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,4])
  mrs2_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,4])
  mrs2_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,4])
  mrs2_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,4])
  mrs2_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,4])
  mrs2_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,4])
  
  t_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,3])
  t_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,3])
  t_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,3])
  t_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,3])
  t_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,3])
  t_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,3])
  t_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,3])
  t_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,3])
  t_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,3])
  t_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,3])
  t_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,3])
  t_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,3])
  
  aa = aa + 200
}

err_mrs1 = err_mrs2 = list(length = num_test)
for (h in 1:num_test){
  err_mrs1[[h]] = (mrs1_h[,h] - t_h[,h])^2
  err_mrs2[[h]] = (mrs2_h[,h] - t_h[,h])^2
}

for(i in 1:num_test){
  MAE_h[7,i] = mean(sqrt(err_mrs1[[i]]))
  MAE_h[8,i] = mean(sqrt(err_mrs2[[i]]))
}
MAE_h

#####MSAR: projecting to power domain
num_test = 12
test.P = truth.p

Pestimations = list(length = 200)
for (ii in 1:200){
  ind = seq(ii,(length(p_mrs1)-200)+ii,200)
  pred_mrs1 = p_mrs1[ind]
  pred_mrs2 = p_mrs2[ind]

  
  tr = truth[ind]
  powerpred_mrs1 = vector(length = length(pred_mrs1))
  powerpred_mrs2 = vector(length = length(pred_mrs2))

  
  powercurve.est = P.est[[ii]]
  powerpred_mrs1[which(pred_mrs1 < vbin[1])] = 0
  powerpred_mrs1[which(pred_mrs1 > vr)] = max(powercurve.est)
  powerpred_mrs2[which(pred_mrs2 < vbin[1])] = 0
  powerpred_mrs2[which(pred_mrs2 > vr)] = max(powercurve.est)
  
  
  #Based on Binning method
  for (i in 1:(length(vbin)-1)){
    powerpred_mrs1[which(pred_mrs1 >= vbin[i] & pred_mrs1 < vbin[i+1])] = powercurve.est[i]
    powerpred_mrs2[which(pred_mrs2 >= vbin[i] & pred_mrs2 < vbin[i+1])] = powercurve.est[i]
    
  }
  
  truetest = test.P[ind]

  mat.out = matrix(ncol=3,nrow=length(truetest))
  mat.out[,1] = powerpred_mrs1
  mat.out[,2] = truetest
  mat.out[,3] = powerpred_mrs2
  Pestimations[[ii]] = mat.out
  if (ii %in% seq(1,200,20)){cat(ii,"  ")}
}

SE_mrs1 = SE_mrs2= 0
for(i in 1:200){
  truetest = Pestimations[[i]][,2]
  out1 = which(truetest < 0)
  out2 = which(is.na(truetest) == T)
  out3 = unique(c(out1,out2))
  
  tempp1 = (Pestimations[[i]][-out3,1] - Pestimations[[i]][-out3,2])^2
  tempp2 = (Pestimations[[i]][-out3,3] - Pestimations[[i]][-out3,2])^2
  
  SE_mrs1 = c(SE_mrs1,tempp1)
  SE_mrs2 = c(SE_mrs2,tempp1)
  
}
SE_mrs1 = SE_mrs1[-1];SE_mrs2 = SE_mrs2[-1]

#Prediction for each horizon
num_test = 12
pest_h_mrs1 = pest_h_mrs2 = list(length = num_test)
for (hh in 1:num_test){
  temp1 =  temp2 = temp3 = 0
  inds = seq(hh,(num_test*num_rolls),num_test)
  for (i in 1:200){
    temp1 = c(temp1,Pestimations[[i]][inds,1])
    temp2 = c(temp2,Pestimations[[i]][inds,2])
    temp3 = c(temp3,Pestimations[[i]][inds,3])
  }
  temp1 = temp1[-1];temp2=temp2[-1];temp3=temp3[-1]
  pest_h_mrs1[[hh]] = temp1
  ptrue_h[[hh]] = temp2
  pest_h_mrs2[[hh]] = temp3
  
  cat(hh,"\n")
}

errpower_mrs1 = errpower_mrs2 = list(length = num_test)
for (h in 1:num_test){
  errpower_mrs1[[h]] = (pest_h_mrs1[[h]] - ptrue_h[[h]])^2
  errpower_mrs2[[h]] = (pest_h_mrs2[[h]] - ptrue_h[[h]])^2
  
}


for(i in 1:num_test){
  out1 = which(ptrue_h[[i]] < 0)
  out2 = which(is.na(ptrue_h[[i]]) == T)
  out = unique(c(out1,out2))
  MAE_hp[7,i] = mean(sqrt(errpower_mrs1[[i]][-out]),na.rm=T)
  MAE_hp[8,i] = mean(sqrt(errpower_mrs2[[i]][-out]),na.rm=T)
}

MAE_hp

Pestimations_mrs = Pestimations
##########################################################################
########################## MSAR results  #################################
##########################################################################
setwd("H:/DingBook/Files Sharing/Code Used in the Book/Chapter4/temp/MSVAR")
load("predout_test.R")
load("indicesout_test.R")
load("paramsout_test.R")
load("paramsout2_test.R")
load("paramsout3_test.R")


#Computing 12-hr average for wind speed
num_test = 12
p_vmrs1 = p_vmrs2  = truth = truth.p = 0

for (i in 1:num_rolls){
  #i = 1
  p_vmrs1 = c(p_vmrs1,as.numeric(pred.out[[i]][,1]))
  p_vmrs2 = c(p_vmrs2,as.numeric(pred.out[[i]][,4]))
  truth = c(truth,as.numeric(pred.out[[i]][,3]))
  
  
  #quantile data
  if(i == 1){p_vmrs1 = p_vmrs1[-1];p_vmrs2 = p_vmrs2[-1];truth = truth[-1]}
  
  
  st.ts = indices.out[[i]][2]+1
  end.ts = st.ts + num_test - 1
  sel.p = datas$Power[ (((st.ts-1)*200)+1): (end.ts*200)]
  truth.p = c(truth.p,sel.p)
  
  if (i %in% seq(1,1400,5)){cat(i,"  ")}
}
truth.p = truth.p[-1]

#Per horizon
num_test = 12
vmrs1_h = vmrs2_h = t_h = matrix(nrow=num_rolls*200,ncol=num_test)
aa = 1
for (i in 1:num_rolls){
  vmrs1_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,1])
  vmrs1_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,1])
  vmrs1_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,1])
  vmrs1_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,1])
  vmrs1_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,1])
  vmrs1_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,1])
  vmrs1_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,1])
  vmrs1_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,1])
  vmrs1_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,1])
  vmrs1_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,1])
  vmrs1_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,1])
  vmrs1_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,1])
  
  vmrs2_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,4])
  vmrs2_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,4])
  vmrs2_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,4])
  vmrs2_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,4])
  vmrs2_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,4])
  vmrs2_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,4])
  vmrs2_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,4])
  vmrs2_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,4])
  vmrs2_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,4])
  vmrs2_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,4])
  vmrs2_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,4])
  vmrs2_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,4])
  
  t_h[aa:(aa+199),1]=as.numeric(pred.out[[i]][1:200,3])
  t_h[aa:(aa+199),2]=as.numeric(pred.out[[i]][201:400,3])
  t_h[aa:(aa+199),3]=as.numeric(pred.out[[i]][401:600,3])
  t_h[aa:(aa+199),4]=as.numeric(pred.out[[i]][601:800,3])
  t_h[aa:(aa+199),5]=as.numeric(pred.out[[i]][801:1000,3])
  t_h[aa:(aa+199),6]=as.numeric(pred.out[[i]][1001:1200,3])
  t_h[aa:(aa+199),7]=as.numeric(pred.out[[i]][1201:1400,3])
  t_h[aa:(aa+199),8]=as.numeric(pred.out[[i]][1401:1600,3])
  t_h[aa:(aa+199),9]=as.numeric(pred.out[[i]][1601:1800,3])
  t_h[aa:(aa+199),10]=as.numeric(pred.out[[i]][1801:2000,3])
  t_h[aa:(aa+199),11]=as.numeric(pred.out[[i]][2001:2200,3])
  t_h[aa:(aa+199),12]=as.numeric(pred.out[[i]][2201:2400,3])
  
  aa = aa + 200
}

err_vmrs1 = err_vmrs2 = list(length = num_test)
for (h in 1:num_test){
  err_vmrs1[[h]] = (vmrs1_h[,h] - t_h[,h])^2
  err_vmrs2[[h]] = (vmrs2_h[,h] - t_h[,h])^2
}

for(i in 1:num_test){
  MAE_h[9,i] = mean(sqrt(err_vmrs1[[i]]))
  MAE_h[10,i] = mean(sqrt(err_vmrs2[[i]]))
}
MAE_h

#####MSVAR: projecting to power domain
num_test = 12
test.P = truth.p
Pestimations = list(length = 200)
for (ii in 1:200){
  ind = seq(ii,(length(p_vmrs1)-200)+ii,200)
  pred_vmrs1 = p_vmrs1[ind]
  pred_vmrs2 = p_vmrs2[ind]
  
  
  tr = truth[ind]
  powerpred_vmrs1 = vector(length = length(pred_vmrs1))
  powerpred_vmrs2 = vector(length = length(pred_vmrs2))

  powercurve.est = P.est[[ii]]
  powerpred_vmrs1[which(pred_vmrs1 < vbin[1])] = 0
  powerpred_vmrs1[which(pred_vmrs1 > vr)] = max(powercurve.est)
  powerpred_vmrs2[which(pred_vmrs2 < vbin[1])] = 0
  powerpred_vmrs2[which(pred_vmrs2 > vr)] = max(powercurve.est)
  
  
  #Based on Binning method
  for (i in 1:(length(vbin)-1)){
    powerpred_vmrs1[which(pred_vmrs1 >= vbin[i] & pred_vmrs1 < vbin[i+1])] = powercurve.est[i]
    powerpred_vmrs2[which(pred_vmrs2 >= vbin[i] & pred_vmrs2 < vbin[i+1])] = powercurve.est[i]
    
  }
  
  truetest = test.P[ind]
  
  mat.out = matrix(ncol=3,nrow=length(truetest))
  mat.out[,1] = powerpred_vmrs1
  mat.out[,2] = truetest
  mat.out[,3] = powerpred_vmrs2
  
  Pestimations[[ii]] = mat.out
  if (ii %in% seq(1,200,20)){cat(ii,"  ")}
}

SE_vmrs1 = SE_vmrs2= 0
for(i in 1:200){
  truetest = Pestimations[[i]][,2]
  out1 = which(truetest < 0)
  out2 = which(is.na(truetest) == T)
  out3 = unique(c(out1,out2))
  
  tempp1 = (Pestimations[[i]][-out3,1] - Pestimations[[i]][-out3,2])^2
  tempp2 = (Pestimations[[i]][-out3,3] - Pestimations[[i]][-out3,2])^2
  
  SE_vmrs1 = c(SE_vmrs1,tempp1)
  SE_vmrs2 = c(SE_vmrs2,tempp1)
  
}
SE_vmrs1 = SE_vmrs1[-1];SE_vmrs2 = SE_vmrs2[-1]

#Prediction for each horizon
num_test = 12
pest_h_vmrs1 = pest_h_vmrs2 = list(length = num_test)
for (hh in 1:num_test){
  temp1 =  temp2 = temp3 = 0
  inds = seq(hh,(num_test*num_rolls),num_test)
  for (i in 1:200){
    temp1 = c(temp1,Pestimations[[i]][inds,1])
    temp2 = c(temp2,Pestimations[[i]][inds,2])
    temp3 = c(temp3,Pestimations[[i]][inds,3])
  }
  
  temp1 = temp1[-1];temp2=temp2[-1];temp3=temp3[-1]
  pest_h_vmrs1[[hh]] = temp1
  ptrue_h[[hh]] = temp2
  pest_h_vmrs2[[hh]] = temp3
  
  cat(hh,"\n")
}

errpower_vmrs1 = errpower_vmrs2 = list(length = num_test)
for (h in 1:num_test){
  errpower_vmrs1[[h]] = (pest_h_vmrs1[[h]] - ptrue_h[[h]])^2
  errpower_vmrs2[[h]] = (pest_h_vmrs2[[h]] - ptrue_h[[h]])^2
  
}


for(i in 1:num_test){
  out1 = which(ptrue_h[[i]] < 0)
  out2 = which(is.na(ptrue_h[[i]]) == T)
  out = unique(c(out1,out2))
  MAE_hp[9,i] = mean(sqrt(errpower_vmrs1[[i]][-out]),na.rm=T)
  MAE_hp[10,i] = mean(sqrt(errpower_vmrs2[[i]][-out]),na.rm=T)
}

MAE_hp

Pestimations_vmrs = Pestimations



##Export final results to Tables
setwd("H:/DingBook/Files Sharing/Code Used in the Book/Chapter4")
MAE_h
colnames(MAE_h) = c("h=1","h=2","h=3","h=4","h=5","h=6","h=7","h=8","h=9","h=10","h=11","h=12")
write.table(MAE_h,file="Table4.4.txt",col.names = T,row.names = T)
MAE_hp
colnames(MAE_hp) = c("h=1","h=2","h=3","h=4","h=5","h=6","h=7","h=8","h=9","h=10","h=11","h=12")
write.table(MAE_hp,file="MAE_power.txt",col.names = T,row.names = T)

##########################################################################
##################### PLOTTING PERCENTAGE IMPROVEMENTS ###################
##########################################################################
par(mfrow=c(1,1),mar=c(6,6,2,2))
IMP = matrix(nrow=9,ncol=12)
marks = c(1,2,3,4,6,7,8,9,10)
for(i in 1:9){
IMP[i,] = ((MAE_h[marks[i],] - MAE_h[5,])/MAE_h[marks[i],])*100
}
rownames(IMP) = c("ASYM","SEP1","SEP2","PER","RSAR","MSAR-s","MSAR-h","MSVAR-s","MSVAR-h")
par(mar=c(5,5,2,2))
plot(seq(1,12,1),rep(10^10,12),panel.first=grid(),xlab=expression(paste("Forecast horizon ",italic(h),sep="")),ylab="% IMP",cex.axis=1.5,cex.lab=2,lwd=3,ylim=c(-3,15))

lines(seq(1,12,1),IMP[1,],cex=3,col="black",lwd=2,pch=19,type="b")
lines(seq(1,12,1),IMP[3,],cex=3,col="red",lwd=3,pch=0,type="b")
lines(seq(1,12,1),IMP[4,],cex=3,col="blue",lwd=3,pch=3,type="b")
lines(seq(1,12,1),IMP[5,],cex=3,col="green",lwd=3,pch=4,type="b")
lines(seq(1,12,1),IMP[6,],cex=3,col="orange",lwd=3,pch=17,type="b")
lines(seq(1,12,1),IMP[8,],cex=3,col="grey",lwd=3,pch=1,type="b")
legend(legend=c("ASYM","SEP","PER","RSAR","MSAR","MSVAR"),cex=1.25,pch=c(19,0,3,4,17,1),lwd=rep(3,6),bty="n",col=c("black","red","blue","green","orange","grey"),"bottomright",y.intersp = .6)
box(lwd=3)
abline(h=0,lty=2,lwd=2,col="black")

IMP2 = matrix(nrow=9,ncol=12)
marks = c(1,2,3,4,6,7,8,9,10)
for(i in 1:9){
  IMP2[i,] = ((MAE_hp[marks[i],] - MAE_hp[5,])/MAE_hp[marks[i],])*100
}
par(mar=c(5,5,2,2))
plot(seq(1,12,1),rep(10^10,12),panel.first=grid(),xlab=expression(paste("Forecast horizon ",italic(h),sep="")),ylab="% IMP",cex.axis=1.5,cex.lab=2,lwd=3,ylim=c(-9,15))
lines(seq(1,12,1),IMP2[1,],cex=3,col="black",lwd=2,pch=19,type="b")
lines(seq(1,12,1),IMP2[3,],cex=3,col="red",lwd=3,pch=0,type="b")
lines(seq(1,12,1),IMP2[4,],cex=3,col="blue",lwd=3,pch=3,type="b")
lines(seq(1,12,1),IMP2[5,],cex=3,col="green",lwd=3,pch=4,type="b")
lines(seq(1,12,1),IMP2[6,],cex=3,col="orange",lwd=3,pch=17,type="b")
lines(seq(1,12,1),IMP2[8,],cex=3,col="grey",lwd=3,pch=1,type="b")
legend(legend=c("ASYM","SEP","PER","RSAR","MSAR","MSVAR"),cex=1.25,pch=c(19,0,3,4,17,1),lwd=rep(3,8),bty="n",col=c("black","red","blue","green","orange","grey"),"bottomright",y.intersp = 0.6)
box(lwd=3)
abline(h=0,lty=2,lwd=2,col="black")