#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(Matrix)
library(circular)
library(markovchain)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
roll.int=6# amount by which we translate every roll
r_speeds = c(0.00, 4.50, 9.00) #refined speed thresholds
r_dirs = c(45,225) #refined direction thresholds
num_regimes = 6 #number of regimes
train_roll = 45 #days

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines
##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

#***********************************************************************
#*******************FORECASTING*****************************************
#***********************************************************************

num_test = 12 #number of hours in forecast horizon
roll = 1 #roll index
end.tr = 719 #end of Phase I
st.tr = 1 #start of Phase I

pred.out  = indices.out  = params.out = params.out2 = list(length = 1)
transmats = list()

repeat{  

  cat(paste("Currently compiling ROLL", roll,"out of 1339 rolls",sep=" "),"\n")
  start_time = Sys.time()
  
  #training and testing data
  train_PP = datas[(( (st.tr-1)*N)+1):(end.tr*N),] #training data
  test_PP = datas[((end.tr*N) + 1):((end.tr*N) + num_test*N),] #testing data
  
  Period = dim(train_PP)[1]/N #number of hours in training data
  
  preds_ar1 = preds_ar2 = preds_ar3 = preds_ar4 = preds_ar5 = preds_ar6 = matrix(ncol=num_test,nrow=N)
  param.ar = matrix(nrow = N*num_regimes,ncol=16) #every 6 rows represent a regime of a turbine
  
  dir.training = dirs[st.tr:end.tr,1:3] #wind direction in training data
  dirav.training = vector(length=dim(dir.training)[1])
  for(jjj in 1:dim(dir.training)[1]){
    dirav.training[jjj] = mean.circular(circular(dir.training[jjj,1:3],type="angles",units="degrees",rotation="clock"),na.rm=T)
    if(dirav.training[jjj] < 0){dirav.training[jjj] = dirav.training[jjj] + 360}
  }
  
  param.ar2 = list(length = N)
  #For each turbine
  
  for (turb.id in 1:N){

    #Turbine-specific speeds
    training = train_PP$Speed[which(train_PP$id == turb.id)]
    
    regimes_sd = vector(length = length(training))
    
    #Labeling observations according to wind regime
    for(ii in 1:length(training)){
      Vav = training[ii]
      if (Vav >= r_speeds[1] & Vav < r_speeds[2]){temp1=1}
      if (Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1=2}
      if (Vav >= r_speeds[3]){temp1=3}
      Dav = dirav.training[ii]
      if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
        temp2 = 1}else{temp2=2}
      if(temp1 == 1 & temp2 == 1){s_t = 1}
      if(temp1 == 1 & temp2 == 2){s_t = 2}
      if(temp1== 2 & temp2 == 1){s_t= 3}
      if(temp1 == 2 & temp2 == 2){s_t = 4}
      if(temp1 == 3 & temp2 == 1){s_t = 5}
      if(temp1 == 3 & temp2 == 2){s_t = 6}
      regimes_sd[ii] = s_t
    }
    
      #Fit a Markov Chain
      chainspeed=markovchainFit(as.factor(regimes_sd),"mle")
      transmat_speed=chainspeed$estimate@transitionMatrix
      transmats[[turb.id]]=transmat_speed
    
    #Determine current regime based on most recent values of wind speed and direction
    Vav = mean(tail(training,1)) #most recently observed wind speed
    if (Vav >= r_speeds[1] & Vav < r_speeds[2]){temp1=1}
    if (Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1=2}
    if (Vav >= r_speeds[3]){temp1=3}
    Dav = tail(dirav.training,1) #most recently observed wind direction
    if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
      temp2 = 1}else{temp2=2}
    if(temp1 == 1 & temp2 == 1){s_t = 1}
    if(temp1 == 1 & temp2 == 2){s_t = 2}
    if(temp1== 2 & temp2 == 1){s_t= 3}
    if(temp1 == 2 & temp2 == 2){s_t = 4}
    if(temp1 == 3 & temp2 == 1){s_t = 5}
    if(temp1 == 3 & temp2 == 2){s_t = 6}
    
    #Regime transitions
    if(s_t==1){x_t = c(1,0,0,0,0,0)}
    if(s_t==2){x_t = c(0,1,0,0,0,0)}
    if(s_t==3){x_t = c(0,0,1,0,0,0)}
    if(s_t==4){x_t = c(0,0,0,1,0,0)}
    if(s_t==5){x_t = c(0,0,0,0,1,0)}
    if(s_t==6){x_t = c(0,0,0,0,0,1)}
    x_tnext = matrix(nrow = num_test,ncol=num_regimes)
    x_tnext[1,]=x_t%*%transmat_speed
    x_tnext[2,]=x_t%*%transmat_speed%*%transmat_speed
    x_tnext[3,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[4,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[5,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[6,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[7,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[8,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[9,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[10,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[11,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    x_tnext[12,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
    s_tnext = vector(length = num_test)
    for(hh in 1:num_test){
      s_tnext[hh] = which(x_tnext[hh,] == max(x_tnext[hh,]))[1]
    }
    
    #pick data peculiar to each regime
    train.reg = list(length = length(r_speeds))
    for(rr in 1:num_regimes){
      Regime = rr
      if(Regime==1 || Regime == 2){reg.ids=which(training>=r_speeds[1] & training<r_speeds[2])}
      if(Regime==3 || Regime == 4){reg.ids=which(training>=r_speeds[2] & training<r_speeds[3])}
      if(Regime==5 || Regime == 6){reg.ids=which(training>=r_speeds[3])}
      regdir.ids = which(dirav.training >= r_dirs[1] & dirav.training < r_dirs[2])
      regdir2.ids = seq(1,length(dirav.training),1)[-regdir.ids]
      if(Regime == 1 || Regime == 3 || Regime == 5){reg.ids = intersect(reg.ids,regdir.ids)}
      if(Regime == 2 || Regime == 4 || Regime == 6){reg.ids = intersect(reg.ids,regdir2.ids)}
      train.reg[[rr]] = training[reg.ids]
    }
    
    #Fit a model for each regime
    p1=p2=p3=matrix(ncol=num_test,nrow=num_regimes)
    for(rr in 1:num_regimes){
      if(length(train.reg[[rr]]) > 1 & length(unique(train.reg[[rr]])) > 1 ){ #to ensure enough data to fit a model
        arfit1 <- try(u <- arima(train.reg[[rr]],order=c(1,0,0),method="ML"), silent = TRUE)
        ERR <- ifelse(class(arfit1) == "try-error", TRUE, FALSE)
        if (ERR == TRUE){
          arfit1 = arima(train.reg[[rr]],order=c(1,0,0),method="CSS")
        }
        p1[rr,]=as.numeric(predict(arfit1,num_test)$pred)
        
        param.ar[(((rr-1)*N)+turb.id),1:4] = c(arfit1$coef[1:2], arfit1$sigma2,mean(train.reg[[rr]]))
        param.ar[(((rr-1)*N)+turb.id),5:16] = p1[rr,] #mean prediction for every horizon
        
        
      } else {p1[rr,] = as.numeric(tail(train.reg[[rr]],1))}
    }
    for(iii in 1:num_test){
      preds_ar1[turb.id,iii] = p1[,iii] %*% x_tnext[iii,] #weighted combination
      preds_ar4[turb.id,iii] = p1[s_tnext[iii],iii] #highest probability
    }
    param.ar2[[turb.id]] = x_tnext
  }
  pred_ar1 = cbind(preds_ar1[,1:num_test])
  pred_ar4 = cbind(preds_ar4[,1:num_test])
  
  testvals1 = as.vector(pred_ar1)
  testvals4 = as.vector(pred_ar4)
  
  #Persistence
  pred_pers = tail(train_PP$Speed,N)
  pred_pers = rep(pred_pers,times=num_test)
  aa = 1;err_pers=vector(length=(num_test*N));rmse_pers=vector(length=num_test)
  for (i in 1:num_test){
    err_pers[aa:(aa+N-1)] = (tail(train_PP$Speed,N) - test_PP$Speed[aa:(aa+N-1)])^2
    rmse_pers[i] = sqrt(mean(err_pers[aa:(aa+N-1)]))
    aa = aa + N
  }
  fin.err_pers = sqrt(mean(err_pers))
  
  #TS models
  testingdata = test_PP$Speed
  fin.err = sqrt(mean((testvals1 - testingdata)^2))
  fin.err4 = sqrt(mean((testvals4 - testingdata)^2))
  
  Final.Predictions = matrix(nrow=(num_test*N),ncol=4)
  Final.Predictions[,1] = testvals1
  Final.Predictions[,2] = pred_pers
  Final.Predictions[,3] = testingdata
  Final.Predictions[,4] = testvals4
  
  #Recording
  pred.out[[roll]] = Final.Predictions
  indices.out[[roll]] = c(st.tr,end.tr)
  params.out[[roll]] = param.ar
  params.out2[[roll]] = param.ar2
  
  #Updating roll
  end.tr=end.tr+roll.int
  st.tr = max(1,end.tr - train_roll*24) #historical data
  if ( (end.tr+12) >= (training_period*24)){break}
  
  #Update
  roll = roll + 1 #uncomment when using repeat
  end_time = Sys.time()
  
  cat(paste("Run time ",end_time - start_time,sep=" "),"\n")
  
}

#Saving outputs
save(pred.out,file="Code Used in the Book/Chapter4/temp/MSAR/predout_test.R")
save(indices.out,file="Code Used in the Book/Chapter4/temp/MSAR/indicesout_test.R")
save(params.out,file="Code Used in the Book/Chapter4/temp/MSAR/paramsout_test.R")
save(params.out2,file="Code Used in the Book/Chapter4/temp/MSAR/paramsout2_test.R")

