#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(Matrix)
library(circular)
library(vars)
library(markovchain)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
roll.int=6# amount by which we translate every roll
r_speeds = c(0.00, 4.50, 9.00) #refined speed thresholds
r_dirs = c(45,225) #refined direction thresholds
num_regimes = 6 #number of regimes
train_roll = 45 #days

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines
##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")


#***********************************************************************
#*******************FORECASTING*****************************************
#***********************************************************************
num_test = 12 #number of hours in forecast horizon
roll = 1 #roll index
end.tr = 719 #end of Phase I
st.tr = 1 #start of Phase I
pred.out = train.out = indices.out  = params.out = params.out2 = params.out3 = list(length = 1)

repeat{
  
  cat(paste("Currently compiling ROLL", roll,"out of 1339 rolls",sep=" "),"\n")
  start_time = Sys.time()
  
  train_PP = datas[(( (st.tr-1)*N)+1):(end.tr*N),] #training data
  test_PP = datas[((end.tr*N) + 1):((end.tr*N) + num_test*N),] #testing data
  dirs_PP = dirs[st.tr:end.tr,1:3] #wind direction training data
  
  
  Period = dim(train_PP)[1]/N #number of hours in training period
  
  predlist = paramlist = list()
  
  #Find spatial averages of wind speed and direction data
  dir.training = dirs[st.tr:end.tr,1:3]
  dirav.training = vector(length=dim(dir.training)[1])
  for(jjj in 1:dim(dir.training)[1]){
    dirav.training[jjj] = mean.circular(circular(dirs[jjj,1:3],type="angles",units="degrees",rotation="clock"),na.rm=T)
    if(dirav.training[jjj] < 0){dirav.training[jjj] = dirav.training[jjj] + 360}
  }
  training = matrix(ncol=N,nrow=Period)
  for (turb.id in 1:N){
    training[,turb.id] = train_PP$Speed[which(train_PP$id == turb.id)] #turbine-specific speeds for the last 30 days
  }
  speed.means = apply(training,FUN = mean,1)
  
  
  regimes_sd = vector(length = dim(training)[1])
  
  
  #Labeling observations according to wind regime
  for(ii in 1:dim(training)[1]){
    Vav = speed.means[ii]
    if (Vav >= r_speeds[1] & Vav < r_speeds[2]){temp1=1}
    if (Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1=2}
    if (Vav >= r_speeds[3]){temp1=3}
    Dav = dirav.training[ii]
    if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
      temp2 = 1}else{temp2=2}
    if(temp1 == 1 & temp2 == 1){s_t = 1}
    if(temp1 == 1 & temp2 == 2){s_t = 2}
    if(temp1 == 2 & temp2 == 1){s_t = 3}
    if(temp1 == 2 & temp2 == 2){s_t = 4}
    if(temp1 == 3 & temp2 == 1){s_t = 5}
    if(temp1 == 3 & temp2 == 2){s_t = 6}
    regimes_sd[ii] = s_t
  }
  
  #Fit one Markov chain for the whole wind farm using spatial averages
  chainspeed=markovchainFit(as.factor(regimes_sd),"mle")
  transmat_speed=chainspeed$estimate@transitionMatrix

  #Find current wind regime based on spatial average of wind speed and direction
  Vav = mean(tail(training,1)) #spatial average of most recently observed wind speeds
  if (Vav >= r_speeds[1] & Vav < r_speeds[2]){temp1=1}
  if (Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1=2}
  if (Vav >= r_speeds[3]){temp1=3}
  Dav = tail(dirav.training,1)
  if(Dav < 0){Dav = Dav + 360}
  Dav = as.numeric(Dav) #spatial average of most recently observed wind direction
  if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
    temp2 = 1}else{temp2=2}
  if(temp1 == 1 & temp2 == 1){s_t = 1}
  if(temp1 == 1 & temp2 == 2){s_t = 2}
  if(temp1== 2 & temp2 == 1){s_t= 3}
  if(temp1 == 2 & temp2 == 2){s_t = 4}
  if(temp1 == 3 & temp2 == 1){s_t = 5}
  if(temp1 == 3 & temp2 == 2){s_t = 6}
  Regime = s_t
  
  #Transition
  if(s_t==1){x_t = c(1,0,0,0,0,0)}
  if(s_t==2){x_t = c(0,1,0,0,0,0)}
  if(s_t==3){x_t = c(0,0,1,0,0,0)}
  if(s_t==4){x_t = c(0,0,0,1,0,0)}
  if(s_t==5){x_t = c(0,0,0,0,1,0)}
  if(s_t==6){x_t = c(0,0,0,0,0,1)}
  num_regimes = length(x_t)
  x_tnext = matrix(nrow = num_test,ncol=num_regimes)
  x_tnext[1,]=x_t%*%transmat_speed
  x_tnext[2,]=x_t%*%transmat_speed%*%transmat_speed
  x_tnext[3,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[4,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[5,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[6,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[7,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[8,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[9,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[10,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[11,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  x_tnext[12,]=x_t%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed%*%transmat_speed
  s_tnext = vector(length = num_test)
  for(hh in 1:num_test){
    s_tnext[hh] = which(x_tnext[hh,] == max(x_tnext[hh,]))[1]
  }
  
  
  #pick data peculiar to each regime
  loc = coords[,3:2]
  spatdist = (rdist.earth(loc,miles=F))#distance 
  
  param.var = matrix(nrow=num_regimes*N,ncol=6*5 + 12)
  param.var.mats = matrix(nrow=N*5,ncol=5)
  #For each wind regime
  for(rr in 1:num_regimes){
    
    preds_var = matrix(ncol=num_test,nrow=N)
    Regime = rr
    if(Regime==1 || Regime == 2){reg.ids=which(speed.means>=r_speeds[1] & speed.means<r_speeds[2])}
    if(Regime==3 || Regime == 4){reg.ids=which(speed.means>=r_speeds[2] & speed.means<r_speeds[3])}
    if(Regime==5 || Regime == 6){reg.ids=which(speed.means>=r_speeds[3])}
    regdir.ids = which(dirav.training >= r_dirs[1] & dirav.training < r_dirs[2])
    regdir2.ids = seq(1,length(dirav.training),1)[-regdir.ids]
    if(Regime == 1 || Regime == 3 || Regime == 5){reg.ids = intersect(reg.ids,regdir.ids)}
    if(Regime == 2 || Regime == 4 || Regime == 6){reg.ids = intersect(reg.ids,regdir2.ids)}
    
    train.reg = matrix(training[reg.ids,],nrow=length(reg.ids),ncol=200)
    
    #Fit a VAR model for each turbine using informative neighborhood
    for(turb in 1:N){
      
      inform_n = head(order(spatdist[,turb]),5) #informative neighborhood
      idd = which(inform_n == turb)
      train.reg.turb = matrix(cbind(train.reg[,inform_n[idd]],train.reg[,inform_n[-idd]]),nrow=dim(train.reg)[1],ncol=length(inform_n))
      train.reg.turb = data.frame(train.reg.turb)
      
      if(dim(train.reg.turb)[1] > 20){ #to ensure reliable estimates  
      varfit1 <- VAR(train.reg.turb,p=1,type="const")
      varpreds = predict(varfit1,n.ahead=12)
      preds_var[turb,] = as.numeric(varpreds$fcst[[1]][,1])
      
      #storing ar coefficients in all dimensions in a matrix that is (R*N,6*N)
      kkk = 1
      for(kk in 1:length(inform_n)){
        param.var[(((rr-1)*N)+turb),(kkk:(kkk+5))] = as.numeric(varfit1$varresult[[kk]]$coefficients)
        kkk = kkk + 6
      }
      param.var[(((rr-1)*N)+turb),31:42] = preds_var[turb,]
      param.var.mats[(1+(turb-1)*5):((1+(turb-1)*5)+4),1:5] = summary(varfit1)$covres
      
      
      }else{preds_var[turb,] = rep(tail(train.reg.turb[,1],1),num_test)}
      
      na.id = which(is.na(preds_var[turb,]) == T)
      if(length(na.id) != 0){
        preds_var[turb,na.id] = tail(train.reg.turb[,1],1)
      }
    }
    predlist[[rr]] = preds_var
    paramlist[[rr]] = param.var.mats #list containing the covariances for every regime
  }
  
  preds_var1 = preds_var2 = matrix(nrow=N,ncol=num_test)
  for(jjj in 1:N){
    p1 = matrix(nrow=num_regimes,ncol=num_test)
    p1[1,] = predlist[[1]][jjj,]
    p1[2,] = predlist[[2]][jjj,]
    p1[3,] = predlist[[3]][jjj,]
    p1[4,] = predlist[[4]][jjj,]
    p1[5,] = predlist[[5]][jjj,]
    p1[6,] = predlist[[6]][jjj,]
    
    for(iii in 1:num_test){
      preds_var1[jjj,iii] = p1[,iii] %*% x_tnext[iii,] #weighted combination
      preds_var2[jjj,iii] = p1[s_tnext[iii],iii] #max probability
    }
  }
  
  pred_var1 = cbind(preds_var1[,1:num_test])
  pred_var2 = cbind(preds_var2[,1:num_test])
  
  testvals1 = as.vector(pred_var1)
  testvals2 = as.vector(pred_var2)
  
  #####Saving predictions####
  #Persistence
  pred_pers = tail(train_PP$Speed,N)
  pred_pers = rep(pred_pers,times=num_test)
  aa = 1;err_pers=vector(length=(num_test*N));rmse_pers=vector(length=num_test)
  for (i in 1:num_test){
    err_pers[aa:(aa+N-1)] = (tail(train_PP$Speed,N) - test_PP$Speed[aa:(aa+N-1)])^2
    rmse_pers[i] = sqrt(mean(err_pers[aa:(aa+N-1)]))
    aa = aa + N
  }
  fin.err_pers = sqrt(mean(err_pers))
  
  #TS models
  testingdata = test_PP$Speed
  fin.err = sqrt(mean((testvals1 - testingdata)^2))
  fin.err2 = sqrt(mean((testvals2 - testingdata)^2))
  
  Final.Predictions = matrix(nrow=(num_test*200),ncol=4)
  Final.Predictions[,1] = testvals1
  Final.Predictions[,2] = pred_pers
  Final.Predictions[,3] = testingdata
  Final.Predictions[,4] = testvals2
  
  pred.out[[roll]] = Final.Predictions
  train.out[[roll]] = train_PP
  indices.out[[roll]] = c(st.tr,end.tr)
  params.out[[roll]] = param.var
  params.out2[[roll]] = paramlist
  params.out3[[roll]] = x_tnext
  
  end.tr=end.tr+roll.int
  st.tr = max(1,end.tr - train_roll*24) #training data
  
  
  if (end.tr >= (training_period*24)){break}
  
  #Updating
  roll = roll + 1 #uncomment when using repeat
  end_time = Sys.time()
  
  cat(paste("Run time ",end_time - start_time,sep=" "),"\n")
  
}


#Saving outputs
save(pred.out,file="Code Used in the Book/Chapter4/temp/MSVAR/predout_test.R")
save(indices.out,file="Code Used in the Book/Chapter4/temp/MSVAR/indicesout_test.R")
save(params.out,file="Code Used in the Book/Chapter4/temp/MSVAR/paramsout_test.R")
save(params.out2,file="Code Used in the Book/Chapter4/temp/MSVAR/paramsout2_test.R")
save(params.out3,file="Code Used in the Book/Chapter4/temp/MSVAR/paramsout3_test.R")
