#########################################################################
#File Description: 
#This file implements the Phase I: 30-day training period to get the 
#raw forecasts
#########################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(circular)
library(fields)
library(Matrix)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter4/funcs/asym_build.R")
source("Code Used in the Book/Chapter4/funcs/asym_fit.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
infpt = 9.5 #inflection point in power curve
vco = 20 #cutout speed 
vr = 13.5 #rated speed
vci = 3.5 #cutin speed
r_dirs = c(0,180) #current direction thresholds
num_regimes = 6 #number of regimes
roll.int= 6 #amount in hours by which forecasts are translated every roll

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 30 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##restructuring data
Phase1_data <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  Phase1_data[aa:(aa+N-1),1] = seq(1,N,1)
  Phase1_data[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  Phase1_data[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
Phase1_data = as.data.frame(Phase1_data)
colnames(Phase1_data) = c("id","Speed","Power")
datas = Phase1_data[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")

dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

#***********************************************************************
#************DEFINING REGIME CHANGES IN TRAINING DATA ******************
#***********************************************************************
#Defining regime partitions and labeling observations
r_speeds = c(0,vci,infpt) #wind speed partitions based on power curve
r_dirs = c(r_dirs[1],r_dirs[2],360) #wind direction partitions based on data visualization

##labeling observations according to the ST-averages of wind speed and direction
aa = 1;speed.means = dir.means = regspeeds = regdirs = regime.means = vector(length = Period)
for (i in 1:length(speed.means)){
  speed.means[i] = mean(datas$Speed[aa:(aa+199)])
  dir.means[i] = mean.circular(circular(dirs[i,1:3],type="angles",units="degrees",rotation="clock"))
  if(dir.means[i] < 0){dir.means[i] = dir.means[i] + 360}
  if(speed.means[i] < r_speeds[2]){regspeeds[i] = 1}
  if(speed.means[i] >= r_speeds[2] & speed.means[i] < r_speeds[3]){regspeeds[i] = 2}
  if(speed.means[i] >= r_speeds[3]){regspeeds[i] = 3}
  if(dir.means[i] >= r_dirs[1] & dir.means[i] < r_dirs[2]){regdirs[i] = 1}
  if(dir.means[i] >= r_dirs[2] & dir.means[i] < r_dirs[3]){regdirs[i] = 2}
  if(regspeeds[i] == 1 & regdirs[i] == 1){regime.means[i] = 1}
  if(regspeeds[i] == 1 & regdirs[i] == 2){regime.means[i] = 2}
  if(regspeeds[i] == 2 & regdirs[i] == 1){regime.means[i] = 3}
  if(regspeeds[i] == 2 & regdirs[i] == 2){regime.means[i] = 4}
  if(regspeeds[i] == 3 & regdirs[i] == 1){regime.means[i] = 5}
  if(regspeeds[i] == 3 & regdirs[i] == 2){regime.means[i] = 6}
  aa = aa + 200
}

#identifying regime changes
diff_reg = vector(length = (length(regime.means) - 1))
for(i in 2:length(regime.means)){
  diff_reg[i] = regime.means[i] - regime.means[i-1]
}
changepoints = which(abs(diff_reg) > 0)

#***********************************************************************
#*******************OBTAINING RAW (UNCALIBRATED) FORECASTS**************
#***********************************************************************
num_test = 12 #no of hours for testing horizon
st.tr = 1;end.tr = 3 #indices of first roll
roll = 1 #roll index
pred.out = train.out = indices.out = fit.out = list(length = 1)

repeat{
  
  cat(paste("Currently compiling ROLL", roll,"out of 118 rolls",sep=" "),"\n")
  start_time = Sys.time()
  
  #Training and testing data
  train_PP = datas[(( (st.tr-1)*N)+1):(end.tr*N),] #training data
  test_PP = datas[((end.tr*N) + 1):((end.tr*N) + num_test*N),] #testing data
  
  #Mast data
  reff1=dirs[st.tr:(end.tr),1];reff1_speed=speed.masts[st.tr:end.tr,1] #mast 1 data
  reff2=dirs[st.tr:(end.tr),2];reff2_speed=speed.masts[st.tr:end.tr,2] #mast 2 data
  reff3=dirs[st.tr:(end.tr),3];reff3_speed=speed.masts[st.tr:end.tr,3] #mast 3 data
  
  #Spatio-temporal average of mast data
  st.av_v = mean(as.numeric(speed.masts[st.tr:end.tr,1:3]),na.rm=T) #speed average
  st.av_d = mean.circular(circular(as.numeric(dirs[st.tr:end.tr,1:3]),type="angles",units="degrees",rotation="clock"),na.rm=TRUE) #dir average
  if(as.numeric(st.av_d) < 0){st.av_d = as.numeric(st.av_d) + 360}
  st.av_d = as.numeric(st.av_d)
  
  #Imputing any missing values in mast data
  #wind direction data
  reff1[which(is.na(reff1)==T)] = st.av_d
  reff2[which(is.na(reff2)==T)] = st.av_d
  reff3[which(is.na(reff3)==T)] = st.av_d
  #wind speed data
  reff1_speed[which(is.na(reff1_speed)==T)] = st.av_v
  reff2_speed[which(is.na(reff2_speed)==T)] = st.av_v
  reff3_speed[which(is.na(reff3_speed)==T)] = st.av_v
  
  #Temporal averages of mast data
  av.dir1 = mean.circular(circular(reff1,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir1) < 0){av.dir1 = as.numeric(av.dir1) + 360}
  av.speed1 = mean(reff1_speed)
  av.dir2 = mean.circular(circular(reff2,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir2) < 0){av.dir2 = as.numeric(av.dir2) + 360}
  av.speed2 = mean(reff2_speed)
  av.dir3 = mean.circular(circular(reff3,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir3) < 0){av.dir3 = as.numeric(av.dir3) + 360}
  av.speed3 = mean(reff3_speed)
  directions = c(as.numeric(av.dir1),as.numeric(av.dir2),as.numeric(av.dir3))
  av.speeds = c(av.speed1,av.speed2,av.speed3)
  
  #Number of time points considered in the training data
  Period = dim(train_PP)[1]/N #Number of hours in the training data
  
  #Grouping turbines according to proximity to masts
  loc_all = matrix( rep( t( coords[,3:2] ) , Period ) , ncol = ncol(coords[,2:3]) , byrow = TRUE )
  loc_t_all = matrix(nrow=dim(train_PP)[1],ncol=2)
  x1 = 0
  for (i in 1:Period){
    x2 = rep(i-1,N)
    x1 = c(x1,x2)
  }
  x1 = x1[-1]
  loc_t_all[,1] = x1
  loc_t_all[,2] = rep(0,dim(train_PP)[1])
  mast_locs = matrix(nrow=num_masts,ncol=2) #mast location data
  mast_locs[1,] = c(coords_mast$lat[1],coords_mast$long[1])
  mast_locs[2,] = c(coords_mast$lat[2],coords_mast$long[2])
  mast_locs[3,] = c(coords_mast$lat[3],coords_mast$long[3])
  areas = vector(length = dim(loc_all)[1])
  mast_dists = rdist.earth(mast_locs[,2:1],loc_all)
  for(qq in 1:dim(loc_all)[1]){
  areas[qq] = which(mast_dists[,qq] == min(mast_dists[,qq]))
  }
  
  #Finding direction partition based on wind direction spatio-temporal average
  ##used only for building asymmetric covariance
  if (st.av_d >= 315){R_theta = "N"}
  if (st.av_d >= 0 & st.av_d < 45){R_theta="N"}
  if(st.av_d >= 45 & st.av_d < 135){R_theta="E"}
  if(st.av_d >= 135 & st.av_d < 225){R_theta="S"}
  if (st.av_d >= 225 & st.av_d < 315){R_theta="W"}

  #Building asymmetric covariance matrix
  asym.build.res = asym_build(reff1,reff2,reff3,reff1_speed,reff2_speed,reff3_speed,R_theta,Period,coords,num_masts,areas,directions,av.speeds)
  Cov.mats = asym.build.res[[1]] #list containing asymmetric covariance matrices for each spatial region
  N.vec = asym.build.res[[2]]
  coords.mats = asym.build.res[[3]]
  Vel = asym.build.res[[4]]
  D_list = asym.build.res[[5]]
  zz_mat = asym.build.res[[6]]
  
  #model fitting
  fit.results = matrix(ncol=10,nrow=num_masts)
  loc_mats = list(length = num_masts)
  loc_t_mats = list(length = num_masts)
  dist_mats = list(length = num_masts)
  tdist_mats = list(length = num_masts)
  z_vecs = list(length = num_masts)
  X_mats = list(length = num_masts)
  opt_sets = list(length = num_masts)
  final.cov.mats = list(length = num_masts)
  mean_est = vector(length = num_masts)
  pred_res = pred_res_sep = list(length = num_masts)
  RMSE_modd = RMSE_modd_sep = vector(length = num_masts)
  train_predictions = test_predictions = list(length = num_masts)
  
  for (modd in 1:num_masts){
    z = train_PP$Speed[which(areas==modd)]*3.6 
    X.m = matrix(nrow=length(z),ncol=1)
    X.m[,1] = rep(0,length(z)) 
    z = z - mean(z) 
    coords_modd = coords.mats[[modd]]
    loc_modd = matrix( rep( t( coords_modd[,3:2] ) , Period ) , ncol = ncol(coords_modd[,2:3]) , byrow = TRUE )
    loc_t_modd = matrix(nrow=dim(loc_modd)[1],ncol=2)
    x1 = 0
    for (i in 1:Period){
      x2 = rep(i-1,N.vec[modd])
      x1 = c(x1,x2)
    }
    x1 = x1[-1]
    loc_t_modd[,1] = x1
    loc_t_modd[,2] = rep(0,dim(loc_modd)[1])
    spatdist_modd = (rdist.earth(loc_modd,miles=F))#spatial distance matrix
    tempdist_modd = rdist(loc_t_modd) #temporal distance matrix
    starting.seed = c(log(var(z)),rep(-1,5),2,-3) #starting point for MLE
    
    asym_signal = 1
    asym.fit.res = asym_fit(z,tempdist_modd,spatdist_modd,1,10^-5,300,starting.seed,0,asym_signal)
    
    #Recovering optimal parameters and other important values
    phi = asym.fit.res[[1]][1]
    nu = asym.fit.res[[1]][2]
    c = asym.fit.res[[1]][3]
    a = asym.fit.res[[1]][4]
    alpha = asym.fit.res[[1]][5]
    delta = asym.fit.res[[1]][6]
    beta = asym.fit.res[[1]][7]
    lambda = asym.fit.res[[1]][8]
    LLs = as.numeric(as.vector(asym.fit.res[[5]]))
    fit.results[modd,] = c(phi,nu,alpha,a,c,delta,beta,lambda,LLs[1],LLs[2])
    loc_mats[[modd]] = loc_modd
    loc_t_mats[[modd]] = loc_t_modd
    dist_mats[[modd]] = spatdist_modd
    tdist_mats[[modd]] = tempdist_modd
    z_vecs[[modd]] = z
    X_mats[[modd]] = X.m
    final.cov.mats[[modd]] = asym.fit.res[[2]]
    S_sep = asym.fit.res[[3]]
    mean_est[modd] = asym.fit.res[[4]][1,1]
    S = asym.fit.res[[2]]
    
    #prediction
    trainvals = vector(length = Period*N.vec[modd])
    K=S
    M = X.m
    Kinv = solve(K)
    K_sep =  S_sep
    Kinv_sep = solve(K_sep)
    testvals = err = testvals_sep = err_sep = vector(length = num_test*N.vec[modd])
    ones = diag(1,nrow=2,ncol=2)
    
    idd = which(areas==modd)
    idd_first = idd[which(idd <= 200)]
    aaaa = 1
    idd_all = vector(length = N.vec[modd]*num_test)
    for (kkk in 1:num_test){
      idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
      aaaa = aaaa + length(idd_first)
    }
    test_modd = test_PP[idd_all,]
    loc_test_modd = do.call(rbind, replicate(num_test, loc_modd[1:N.vec[modd],], simplify=FALSE))
    loc_t_test = matrix(nrow=dim(loc_test_modd)[1],ncol=2)
    ccc = 1
    for (i in 1:num_test){
      loc_t_test[ccc:(ccc+N.vec[modd]-1),1] =  rep(tail(loc_t_modd,1)[1] + i,N.vec[modd])
      ccc = ccc + N.vec[modd]
    }
    loc_t_test[,2] = rep(0,num_test*N.vec[modd])
    
    for (i in 1:length(testvals)){
      m = 0
      test.coords = loc_test_modd[i,]
      test.t = loc_t_test[i,]
      t1 = matrix(nrow=1,ncol=2)
      t1[1,] = test.coords
      sdist_vec = as.vector(rdist.earth(t1,loc_modd,miles=F))
      t1 = matrix(nrow=1,ncol=2)
      t1[1,] = test.t
      tdist_vec = as.vector(rdist(t1,loc_t_modd))
      k1 = (1-nu)/(1 + a* (tdist_vec)^(2*alpha))
      k2 = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(beta/2)))
      k2[which(sdist_vec == 0)] = k2[which(sdist_vec == 0)] + (nu/(1-nu))
      k12 = k1 * k2
      k2_sep = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(0/2)))
      k2_sep[which(sdist_vec == 0)] = k2_sep[which(sdist_vec == 0)] + (nu/(1-nu))
      k12_sep = k1 * k2_sep
      r = vector(length = length(loc_modd[,1]))
      for (j in 1:length(loc_modd[,1])){
        
        V = Vel[modd,]
        D = D_list[[modd]]
        zz = zz_mat[modd,]
        
        h2 = loc_test_modd[i,2] - loc_modd[j,2] #diff in lat
        h1 = loc_test_modd[i,1] - loc_modd[j,1] #diff in lon
        hh = c(h1,h2)
        tt = loc_t_test[i,1] - loc_t_modd[j,1]
        trans= matrix(hh + tt*V*zz,nrow=1,ncol=2)
        r[j] = (1/sqrt(det(ones + ((tt)^2)*D)))*exp(-((as.numeric(trans %*% solve(ones + ((tt)^2)*D) %*% t(trans)))))
      }
      
      
      k = phi * ( ((1-lambda)*k12) + (lambda*r))
      k = as.matrix(k,nrow=dim(term1a)[2],ncol=1)
      
      k_sep = phi * k12_sep
      k_sep = as.matrix(k_sep,nrow=dim(term1a)[2],ncol=1)

      lam = Kinv %*% k  
      lam2 = Kinv_sep %*% k_sep
      
      obs = as.matrix(z)
      pred = (t(lam) %*% obs) 
      pred_sep = (t(lam2) %*% obs) 
      
      pred = pred + mean(train_PP$Speed[which(areas==modd)]*3.6)
      pred_sep = pred_sep + mean(train_PP$Speed[which(areas==modd)]*3.6)
      
      testvals[i] = pred/3.6
      err[i] = (testvals[i] - test_modd$Speed[i])^2
      testvals_sep[i] = pred_sep/3.6
      err_sep[i] = (testvals_sep[i] - test_modd$Speed[i])^2
    }
    test_predictions[[modd]] = testvals
    RMSE_asym = sqrt(mean(err))
    RMSE_sep = sqrt(mean(err_sep))
    
    RMSE_modd[modd] = RMSE_asym
    RMSE_modd_sep[modd] = RMSE_sep
    preds = matrix(nrow=length(testvals),ncol=2)
    preds[,1] = testvals
    preds[,2] = test_modd$Speed
    preds_sep = matrix(nrow=length(testvals_sep),ncol=2)
    preds_sep[,1] = testvals_sep
    preds_sep[,2] = test_modd$Speed
    
    pred_res[[modd]] = preds
    pred_res_sep[[modd]] = preds_sep
  }
  
  #Persistence PER
  pred_pers = tail(train_PP$Speed,200)
  pred_pers = rep(pred_pers,times=num_test)
  aa = 1;err_pers=vector(length=(num_test*200));rmse_pers=vector(length=num_test)
  for (i in 1:num_test){
    err_pers[aa:(aa+199)] = (tail(train_PP$Speed,200) - test_PP$Speed[aa:(aa+199)])^2
    rmse_pers[i] = sqrt(mean(err_pers[aa:(aa+199)]))
    aa = aa + 200
  }
  fin.err_pers = sqrt(mean(err_pers))
  
  #ASYM & SEP
  for (modd in 1:num_masts){
    idd = which(areas==modd)
    idd_first = idd[which(idd <= 200)]
    aaaa = 1
    idd_all = vector(length = N.vec[modd]*num_test)
    for (kkk in 1:num_test){
      idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
      aaaa = aaaa + length(idd_first)
    }
    if(modd==1){id_1=idd_all}
    if(modd==2){id_2=idd_all}
    if(modd==3){id_3=idd_all}
  }
  fin.res = matrix(nrow=(num_test*200),ncol=2)
  fin.res[id_1,1:2] = pred_res[[1]]
  fin.res[id_2,1:2] = pred_res[[2]]
  fin.res[id_3,1:2] = pred_res[[3]]
  fin.res.sep = matrix(nrow=(num_test*200),ncol=2)
  fin.res.sep[id_1,1:2] = pred_res_sep[[1]]
  fin.res.sep[id_2,1:2] = pred_res_sep[[2]]
  fin.res.sep[id_3,1:2] = pred_res_sep[[3]]
  fin.err = sqrt(mean((fin.res[,1] - fin.res[,2])^2))
  fin.err.sep = sqrt(mean((fin.res.sep[,1] - fin.res.sep[,2])^2))
  
  Final.Predictions = matrix(nrow=(num_test*200),ncol=4)
  Final.Predictions[,1] = fin.res[,1]
  Final.Predictions[,2] = pred_pers
  Final.Predictions[,3] = fin.res[,2]
  Final.Predictions[,4] = fin.res.sep[,1]
  
  pred.out[[roll]] = Final.Predictions
  train.out[[roll]] = train_PP
  fit.out[[roll]] = fit.results
  indices.out[[roll]] = c(st.tr,end.tr)
  
  #Sliding rolling interval
  temppp = end.tr
  end.tr = end.tr + roll.int
  
  if ( (end.tr+12) >= (training_period*24)){break}
  
  
  #Determining new starting point
  most.recent.CP = tail(changepoints[which(changepoints <= end.tr)],1)
  st.tr = min(most.recent.CP,end.tr-1) #at least 2 hours for training of an asymmetric model
  st.tr = max(st.tr,end.tr-3) #at most 4 hours for training as decided by PACF plots
  
  #Updating roll counter
  roll = roll + 1 
  
  end_time = Sys.time()
  cat(paste("Roll Run time is ",end_time - start_time,sep=" "),"\n") #Running time of a forecasting roll
}

#Saving outputs
save(pred.out,file="Code Used in the Book/Chapter4/temp/predout.R")
save(train.out,file="Code Used in the Book/Chapter4/temp/trainout.R")
save(indices.out,file="Code Used in the Book/Chapter4/temp/indicesout.R")
save(changepoints,file="Code Used in the Book/Chapter4/temp/changepoints.R")


