#########################################################################
#File Description: 
#This file implements the Phase II: Learnining the calibration functions
#########################################################################

#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(circular)
library(fields)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly
##Functions to import
source("Code Used in the Book/Chapter4/funcs/bin_est.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
infpt = 9.5 #inflection point in power curve
vco = 20 #cutout speed 
vr = 13.5 #rated speed
vci = 3.5 #cutin speed
r_dirs = c(0,180) #current direction thresholds
num_regimes = 6 #number of regimes

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 30 #days
Period = training_period*24 #hours
N = 200 #number of turbines

##restructuring data
Phase1_data <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  Phase1_data[aa:(aa+N-1),1] = seq(1,N,1)
  Phase1_data[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  Phase1_data[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
Phase1_data = as.data.frame(Phase1_data)
colnames(Phase1_data) = c("id","Speed","Power")
datas = Phase1_data[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")

dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

load("Code Used in the Book/Chapter4/temp/trainout.R")
load("Code Used in the Book/Chapter4/temp/predout.R")
load("Code Used in the Book/Chapter4/temp/indicesout.R")
load("Code Used in the Book/Chapter4/temp/changepoints.R")

##First Estimate wind power curves using binning method
output.list = bin_est(datas,bw=.5,cutin = vci,rated = vr,numturbs = N)
vbin = as.numeric(output.list[[1]])
P.est = output.list[[2]]
num_rolls = length(pred.out)

#candidate regimes
cands = expand.grid(seq(vci,vci+1.5,0.5),seq(infpt-1.5,infpt,0.5),seq(180-45,180+45,15))
poss_regimes = matrix(nrow=dim(cands)[1],ncol=dim(cands)[2]+2)
poss_regimes[,1] = rep(0,dim(cands)[1])
poss_regimes[,2:3] = as.matrix(cands[,1:2],ncol=2)
poss_regimes[,4] = as.matrix(cands[,3]-180,ncol=1)
poss_regimes[,5] = as.matrix(cands[,3],ncol=1)
canderrs = matrix(nrow=dim(poss_regimes)[1],ncol=3)
num_rolls = length(pred.out)
num_test = 12

for(regcand in 1:dim(cands)[1]){
  r_speeds = poss_regimes[regcand,1:3] #wind speed partitions
  r_dirs = c(poss_regimes[regcand,4:5]) #wind direction partitions
  reg1.data=reg2.data=reg3.data=reg4.data=reg5.data=reg6.data=matrix(nrow=1,ncol=8) #for storing each regime's data
  rlt1 = rlt2 = rlt3 = rlt4 = rlt5 = rlt6 = vector(length=1) #for storing each regime's runlengths
  regime.means = vector(length = num_rolls)
  for (roll in 1:num_rolls){
    V = mean(train.out[[roll]]$Speed,na.rm=T) #Finding spatio-temporal average wind speed of roll
    
    #Defining start and end of training & testing periods
    end.tr = indices.out[[roll]][2]
    st.tr = indices.out[[roll]][1]
    st.ts = end.tr + 1
    end.ts = st.ts + num_test - 1
    
    D = mean.circular(circular(dirs[(st.tr:end.tr),1:3],type="angles",units="degrees",rotation="clock"),na.rm=T) #ST-average of wind direction of that roll
    if(D < 0){D = D + 360}
    D = as.numeric(D)
    
    #Labeling regimes
    if(V < r_speeds[2]){temp1 = 1}
    if(V >= r_speeds[2] & V < r_speeds[3]){temp1 = 2}
    if(V >= r_speeds[3]){temp1 = 3}
    if(r_dirs[1] >= 0){
      if(D >= r_dirs[1] & D < r_dirs[2]){
        temp2 = 1}else{temp2=2}
    }
    if(r_dirs[1] < 0){
      if(D >= (r_dirs[1] + 360)){temp2=1}
      if(D < r_dirs[2]){temp2=1}
      if(D >= r_dirs[2]){temp2 = 2}
    }
    if(temp1 == 1 & temp2 == 1){regime.means[roll] = 1}
    if(temp1 == 1 & temp2 == 2){regime.means[roll] = 2}
    if(temp1== 2 & temp2 == 1){regime.means[roll] = 3}
    if(temp1 == 2 & temp2 == 2){regime.means[roll] = 4}
    if(temp1 == 3 & temp2 == 1){regime.means[roll] = 5}
    if(temp1 == 3 & temp2 == 2){regime.means[roll] = 6}
    R = regime.means[roll]
    
    #The most recently observed changepoint
    last.cp = tail(changepoints[which(changepoints < end.tr)],1) #finding the most recent CP up to time t
    if(length(last.cp) == 0){last.cp = 1}
    
    rl = seq(st.ts-last.cp,(st.ts-last.cp)+num_test-1,1) #Computing runlength x(t+h)
    h = seq(1,num_test,1) #Forecast horizon
    
    true_vals = pred.out[[roll]][,3] #True values of test
    asym_preds = pred.out[[roll]][,1] #ASYM predictions
    pers_preds = pred.out[[roll]][,2] #PER predictions
    true_vals_power = datas$Power[(((st.ts-1)*200)+1):(end.ts*200)] #Power true values of power
    
    #Converting ASYM & PER forecasts to power estimates using Power curve
    asym_power = pers_power = vector(length = length(asym_preds))
    for (ii in 1:200){
      ind = seq(ii,(length(asym_preds)-200)+ii,200)
      pred_a = asym_preds[ind]
      pred_p = pers_preds[ind]
      powercurve.est = P.est[[ii]]
      temp1 = temp2 = vector(length = length(pred_a))
      temp1[which(pred_a < vbin[1])] = 0
      temp2[which(pred_p < vbin[1])] = 0
      temp1[which(pred_a > vr)] = tail(powercurve.est,1)
      temp2[which(pred_p > vr)] = tail(powercurve.est,1)
      for (i in 1:(length(vbin)-1)){
        temp1[which(pred_a >= vbin[i] & pred_a < vbin[i+1])] = powercurve.est[i]
        temp2[which(pred_p >= vbin[i] & pred_p < vbin[i+1])] = powercurve.est[i]
      }
      asym_power[ind] = temp1
      pers_power[ind] = temp2
    }
    
    rl_t = rl
    rl = rep(rl,each=200)
    hh = rep(h,each=200)
    if(R==1){reg1.data = as.data.frame(rbind(reg1.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt1 = c(rlt1,rl_t)}
    if(R==2){reg2.data = as.data.frame(rbind(reg2.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt2 = c(rlt2,rl_t)}
    if(R==3){reg3.data = as.data.frame(rbind(reg3.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt3 = c(rlt3,rl_t)}
    if(R==4){reg4.data = as.data.frame(rbind(reg4.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt4=c(rlt4,rl_t)}
    if(R==5){reg5.data = as.data.frame(rbind(reg5.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt5 = c(rlt5,rl_t)}
    if(R==6){reg6.data = as.data.frame(rbind(reg6.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt6=c(rlt6,rl_t)}
    
  }
  reg1.data = reg1.data[-1,];reg2.data = reg2.data[-1,];reg3.data = reg3.data[-1,]
  reg4.data = reg4.data[-1,];reg5.data = reg5.data[-1,];reg6.data = reg6.data[-1,]
  rlt1 = rlt1[-1];rlt2=rlt2[-1];rlt3=rlt3[-1];rlt4=rlt4[-1];rlt5=rlt5[-1];rlt6=rlt6[-1]
  
  #Speed error
  mse = mean(c(((reg1.data[,1] - reg1.data[,2])^2),((reg2.data[,1] - reg2.data[,2])^2),((reg3.data[,1] - reg3.data[,2])^2),((reg4.data[,1] - reg4.data[,2])^2),((reg5.data[,1] - reg5.data[,2])^2),((reg6.data[,1] - reg6.data[,2])^2)))
  
  regimes.data = list(length = num_regimes)
  regimes.data[[1]] = reg1.data;regimes.data[[2]] = reg2.data;regimes.data[[3]] = reg3.data
  regimes.data[[4]] = reg4.data;regimes.data[[5]] = reg5.data;regimes.data[[6]] = reg6.data
  rlt.data = list(length = num_regimes)
  rlt.data[[1]]=rlt1;rlt.data[[2]]=rlt2;rlt.data[[3]]=rlt3;rlt.data[[4]]=rlt4;rlt.data[[5]]=rlt5;rlt.data[[6]]=rlt6
  
  #Estimating calibration function using lognorm functional form
  rmse = vector(length = num_regimes)
  ests = matrix(nrow=num_regimes,ncol=3)
  for (reg in 1:num_regimes){
    obj_fn <- function(inputs){
      theta_h = inputs[1:3]
      theta_h[2:3] = exp(theta_h[2:3])
      runlengths = rlt.data[[reg]]
      asym_preds = regimes.data[[reg]][,2]
      true_vals = regimes.data[[reg]][,1]
      betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
      betas = rep(betas_t,each=200)
      f_adj = asym_preds + betas
      err = mean(((true_vals - f_adj)^2))
      return(err)
    }
    LSE = nlm(obj_fn,p=rep(0,3),iterlim = 1000,stepmax = 1,steptol = 10^-10)
    rmse[reg] = sqrt(LSE$minimum)
    ests[reg,] = LSE$estimate
  }
  ests[,2:3] = exp(ests[,2:3])
  
  #Converting calibrated forecasts to power estimates
  err.p = vector(length = num_regimes)
  for (reg in 1:num_regimes){
    theta_h = ests[reg,]
    runlengths = rlt.data[[reg]]#regimes.data[[reg]][,3]
    asym_preds = regimes.data[[reg]][,2]
    true_vals = regimes.data[[reg]][,1]
    betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
    #betas_t = theta_h[1] * exp(runlengths * theta_h[2])
    #betas_t = theta_h[1] * (runlengths^theta_h[2])
    
    betas = rep(betas_t,each=200)
    f_adj = asym_preds + betas
    true_vals_p = regimes.data[[reg]][,5]
    
    f_vals_p = vector(length = length(f_adj))
    for (ii in 1:N){
      ind = seq(ii,(length(f_adj)-N)+ii,N)
      tempp = f_adj[ind]
      temppp  = vector(length = length(tempp))
      powercurve.est = P.est[[ii]]
      temppp[which(tempp < vbin[1])] = 0
      temppp[which(tempp > vr)] = tail(powercurve.est,1)
      for (i in 1:(length(vbin)-1)){
        temppp[which(tempp >= vbin[i] & tempp < vbin[i+1])] = powercurve.est[i]
      }
      f_vals_p[ind] = temppp
    }
    out1 = which(true_vals_p < 0)
    out2 = which(is.na(true_vals_p) == T)
    out3 = unique(c(out1,out2))
    err.p[reg] = mean(((true_vals_p[-out3] - f_vals_p[-out3])^2))
  }
  
  #Storing final adjusted forecasts
  lognorm_p = lognorm_t = list(length = num_regimes)
  for (rr in 1:num_regimes){
    theta_h = ests[rr,]
    runlengths = rlt.data[[rr]]
    asym_preds = regimes.data[[rr]][,2]
    true_vals = regimes.data[[rr]][,1]
    betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
    #betas_t = theta_h[1] * exp(theta_h[2]*runlengths)
    #betas_t = theta_h[1] * (runlengths^theta_h[2])
    
    betas = rep(betas_t,each=N)
    f_adj = asym_preds + betas
    lognorm_p[[rr]] = f_adj
    lognorm_t[[rr]]= true_vals
  }
  lp = as.numeric(as.vector(unlist(lognorm_p)))
  lt = as.numeric(as.vector(unlist(lognorm_t)))
  e_lognorm = sqrt(mean((lp - lt)^2))
  
  lognorm_p = lognorm_t = list(length = num_regimes)
  for (rr in 1:num_regimes){
    theta_h = ests[rr,]
    runlengths = rlt.data[[rr]]
    asym_preds = regimes.data[[rr]][,2]
    true_vals = regimes.data[[rr]][,1]
    betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
    #betas_t = theta_h[1] * exp(theta_h[2]*runlengths)
    #betas_t = theta_h[1] * (runlengths^theta_h[2])
    betas = rep(betas_t,each=200)
    f_adj = asym_preds + betas
    
    true_vals_p = regimes.data[[rr]][,5]
    f_vals_p = vector(length = length(f_adj))
    for (ii in 1:200){
      ind = seq(ii,(length(f_adj)-200)+ii,200)
      tempp = f_adj[ind]
      temppp  = vector(length = length(tempp))
      powercurve.est = P.est[[ii]]
      temppp[which(tempp < vbin[1])] = 0
      temppp[which(tempp > vr)] = tail(powercurve.est,1)
      for (i in 1:(length(vbin)-1)){
        temppp[which(tempp >= vbin[i] & tempp < vbin[i+1])] = powercurve.est[i]
      }
      f_vals_p[ind] = temppp
    }
    
    out1 = which(true_vals_p < 0)
    out2 = which(is.na(true_vals_p) == T)
    out3 = unique(c(out1,out2))
    
    lognorm_p[[rr]] = f_vals_p[-out3]
    lognorm_t[[rr]]= true_vals_p[-out3]
  }
  lp = as.numeric(as.vector(unlist(lognorm_p)))
  lt = as.numeric(as.vector(unlist(lognorm_t)))
  e_lognorm_p = sqrt(mean((lp - lt)^2))
  
  canderrs[regcand,1:2] = c(e_lognorm,e_lognorm_p) #Speed and power errors
  canderrs[regcand,3] = ((e_lognorm - sqrt(mse))/sqrt(mse))*-100 #% Imp. in Speed
  cat(regcand,"\n")
}

#Calibration Functions Using optimal regime cutoffs
opt_part = which(canderrs[,1] == min(canderrs[,1]))
r_speeds = poss_regimes[opt_part,1:3] #wind speed partitions
r_dirs = poss_regimes[opt_part,4:5] #wind direction partitions
reg1.data=reg2.data=reg3.data=reg4.data=reg5.data=reg6.data=matrix(nrow=1,ncol=8) #for storing each regime's data
rlt1 = rlt2 = rlt3 = rlt4 = rlt5 = rlt6 = vector(length=1) #for storing each regime's runlengths
regime.means = vector(length = num_rolls)
for (roll in 1:num_rolls){
  V = mean(train.out[[roll]]$Speed,na.rm=T) #Finding spatio-temporal average wind speed of roll
  
  #Defining start and end of training & testing periods
  end.tr = indices.out[[roll]][2]
  st.tr = indices.out[[roll]][1]
  st.ts = end.tr + 1
  end.ts = st.ts + num_test - 1
  
  D = mean.circular(circular(dirs[(st.tr:end.tr),1:3],type="angles",units="degrees",rotation="clock"),na.rm=T) #ST-average of wind direction of that roll
  if(D < 0){D = D + 360}
  D = as.numeric(D)
  
  #Labeling regimes
  if(V < r_speeds[2]){temp1 = 1}
  if(V >= r_speeds[2] & V < r_speeds[3]){temp1 = 2}
  if(V >= r_speeds[3]){temp1 = 3}
  if(r_dirs[1] >= 0){
    if(D >= r_dirs[1] & D < r_dirs[2]){
      temp2 = 1}else{temp2=2}
  }
  if(r_dirs[1] < 0){
    if(D >= (r_dirs[1] + 360)){temp2=1}
    if(D < r_dirs[2]){temp2=1}
    if(D >= r_dirs[2]){temp2 = 2}
  }
  if(temp1 == 1 & temp2 == 1){regime.means[roll] = 1}
  if(temp1 == 1 & temp2 == 2){regime.means[roll] = 2}
  if(temp1== 2 & temp2 == 1){regime.means[roll] = 3}
  if(temp1 == 2 & temp2 == 2){regime.means[roll] = 4}
  if(temp1 == 3 & temp2 == 1){regime.means[roll] = 5}
  if(temp1 == 3 & temp2 == 2){regime.means[roll] = 6}
  R = regime.means[roll]
  
  
  #The most recently observed changepoint
  last.cp = tail(changepoints[which(changepoints < end.tr)],1) #finding the most recent CP up to time t
  if(length(last.cp) == 0){last.cp = 1}
  
  rl = seq(st.ts-last.cp,(st.ts-last.cp)+num_test-1,1) #Computing runlength x(t+h)
  h = seq(1,num_test,1) #Forecast horizon
  
  true_vals = pred.out[[roll]][,3] #True values of test
  asym_preds = pred.out[[roll]][,1] #ASYM predictions
  pers_preds = pred.out[[roll]][,2] #PER predictions
  true_vals_power = datas$Power[(((st.ts-1)*200)+1):(end.ts*200)] #Power true values of power
  
  #Converting ASYM & PER forecasts to power estimates using Power curve
  asym_power = pers_power = vector(length = length(asym_preds))
  for (ii in 1:N){
    ind = seq(ii,(length(asym_preds)-N)+ii,N)
    pred_a = asym_preds[ind]
    pred_p = pers_preds[ind]
    powercurve.est = P.est[[ii]]
    temp1 = temp2 = vector(length = length(pred_a))
    temp1[which(pred_a < vbin[1])] = 0
    temp2[which(pred_p < vbin[1])] = 0
    temp1[which(pred_a > vr)] = tail(powercurve.est,1)
    temp2[which(pred_p > vr)] = tail(powercurve.est,1)
    for (i in 1:(length(vbin)-1)){
      temp1[which(pred_a >= vbin[i] & pred_a < vbin[i+1])] = powercurve.est[i]
      temp2[which(pred_p >= vbin[i] & pred_p < vbin[i+1])] = powercurve.est[i]
    }
    asym_power[ind] = temp1
    pers_power[ind] = temp2
  }
  
  rl_t = rl
  rl = rep(rl,each=200)
  hh = rep(h,each=200)
  if(R==1){reg1.data = as.data.frame(rbind(reg1.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt1 = c(rlt1,rl_t)}
  if(R==2){reg2.data = as.data.frame(rbind(reg2.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt2 = c(rlt2,rl_t)}
  if(R==3){reg3.data = as.data.frame(rbind(reg3.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt3 = c(rlt3,rl_t)}
  if(R==4){reg4.data = as.data.frame(rbind(reg4.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt4=c(rlt4,rl_t)}
  if(R==5){reg5.data = as.data.frame(rbind(reg5.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt5 = c(rlt5,rl_t)}
  if(R==6){reg6.data = as.data.frame(rbind(reg6.data,cbind(true_vals,asym_preds,rl,pers_preds,true_vals_power,asym_power,pers_power,hh)));rlt6=c(rlt6,rl_t)}
  
}
reg1.data = reg1.data[-1,];reg2.data = reg2.data[-1,];reg3.data = reg3.data[-1,]
reg4.data = reg4.data[-1,];reg5.data = reg5.data[-1,];reg6.data = reg6.data[-1,]
rlt1 = rlt1[-1];rlt2=rlt2[-1];rlt3=rlt3[-1];rlt4=rlt4[-1];rlt5=rlt5[-1];rlt6=rlt6[-1]

#Speed error
mse = mean(c(((reg1.data[,1] - reg1.data[,2])^2),((reg2.data[,1] - reg2.data[,2])^2),((reg3.data[,1] - reg3.data[,2])^2),((reg4.data[,1] - reg4.data[,2])^2),((reg5.data[,1] - reg5.data[,2])^2),((reg6.data[,1] - reg6.data[,2])^2)))

regimes.data = list(length = num_regimes)
regimes.data[[1]] = reg1.data;regimes.data[[2]] = reg2.data;regimes.data[[3]] = reg3.data
regimes.data[[4]] = reg4.data;regimes.data[[5]] = reg5.data;regimes.data[[6]] = reg6.data
rlt.data = list(length = num_regimes)
rlt.data[[1]]=rlt1;rlt.data[[2]]=rlt2;rlt.data[[3]]=rlt3;rlt.data[[4]]=rlt4;rlt.data[[5]]=rlt5;rlt.data[[6]]=rlt6

#Learning calibration function
rmse = vector(length = num_regimes)
ests = matrix(nrow=num_regimes,ncol=3)
for (reg in 1:num_regimes){
  obj_fn <- function(inputs){
    theta_h = inputs[1:3]
    theta_h[2:3] = exp(theta_h[2:3])
    runlengths = rlt.data[[reg]]
    asym_preds = regimes.data[[reg]][,2]
    true_vals = regimes.data[[reg]][,1]
    betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
    betas = rep(betas_t,each=200)
    f_adj = asym_preds + betas
    err = mean(((true_vals - f_adj)^2))
    return(err)
  }
  LSE = nlm(obj_fn,p=rep(0,3),iterlim = 1000,stepmax = 1,steptol = 10^-10)
  rmse[reg] = sqrt(LSE$minimum)
  ests[reg,] = LSE$estimate
}
ests[,2:3] = exp(ests[,2:3])

#Converting calibrated forecasts to power estimates
err.p = vector(length = num_regimes)
for (reg in 1:num_regimes){
  theta_h = ests[reg,]
  runlengths = rlt.data[[reg]]#regimes.data[[reg]][,3]
  asym_preds = regimes.data[[reg]][,2]
  true_vals = regimes.data[[reg]][,1]
  betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
  #betas_t = theta_h[1] * exp(runlengths * theta_h[2])
  #betas_t = theta_h[1] * (runlengths^theta_h[2])
  
  betas = rep(betas_t,each=200)
  f_adj = asym_preds + betas
  true_vals_p = regimes.data[[reg]][,5]
  
  f_vals_p = vector(length = length(f_adj))
  for (ii in 1:200){
    ind = seq(ii,(length(f_adj)-200)+ii,200)
    tempp = f_adj[ind]
    temppp  = vector(length = length(tempp))
    powercurve.est = P.est[[ii]]
    temppp[which(tempp < vbin[1])] = 0
    temppp[which(tempp > vr)] = tail(powercurve.est,1)
    for (i in 1:(length(vbin)-1)){
      temppp[which(tempp >= vbin[i] & tempp < vbin[i+1])] = powercurve.est[i]
    }
    f_vals_p[ind] = temppp
  }
  out1 = which(true_vals_p < 0)
  out2 = which(is.na(true_vals_p) == T)
  out3 = unique(c(out1,out2))
  err.p[reg] = mean(((true_vals_p[-out3] - f_vals_p[-out3])^2))
}

#Storing final adjusted forecasts
lognorm_p = lognorm_t = list(length = num_regimes)
for (rr in 1:num_regimes){
  theta_h = ests[rr,]
  runlengths = rlt.data[[rr]]
  asym_preds = regimes.data[[rr]][,2]
  true_vals = regimes.data[[rr]][,1]
  betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
  #betas_t = theta_h[1] * exp(theta_h[2]*runlengths)
  #betas_t = theta_h[1] * (runlengths^theta_h[2])
  
  betas = rep(betas_t,each=200)
  f_adj = asym_preds + betas
  lognorm_p[[rr]] = f_adj
  lognorm_t[[rr]]= true_vals
}
lp = as.numeric(as.vector(unlist(lognorm_p)))
lt = as.numeric(as.vector(unlist(lognorm_t)))
e_lognorm = sqrt(mean((lp - lt)^2))

lognorm_p = lognorm_t = list(length = num_regimes)
for (rr in 1:num_regimes){
  theta_h = ests[rr,]
  runlengths = rlt.data[[rr]]
  asym_preds = regimes.data[[rr]][,2]
  true_vals = regimes.data[[rr]][,1]
  betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
  #betas_t = theta_h[1] * exp(theta_h[2]*runlengths)
  #betas_t = theta_h[1] * (runlengths^theta_h[2])
  betas = rep(betas_t,each=200)
  f_adj = asym_preds + betas
  
  true_vals_p = regimes.data[[rr]][,5]
  f_vals_p = vector(length = length(f_adj))
  for (ii in 1:200){
    ind = seq(ii,(length(f_adj)-200)+ii,200)
    tempp = f_adj[ind]
    temppp  = vector(length = length(tempp))
    powercurve.est = P.est[[ii]]
    temppp[which(tempp < vbin[1])] = 0
    temppp[which(tempp > vr)] = tail(powercurve.est,1)
    for (i in 1:(length(vbin)-1)){
      temppp[which(tempp >= vbin[i] & tempp < vbin[i+1])] = powercurve.est[i]
    }
    f_vals_p[ind] = temppp
  }
  
  out1 = which(true_vals_p < 0)
  out2 = which(is.na(true_vals_p) == T)
  out3 = unique(c(out1,out2))
  
  lognorm_p[[rr]] = f_vals_p[-out3]
  lognorm_t[[rr]]= true_vals_p[-out3]
}
lp = as.numeric(as.vector(unlist(lognorm_p)))
lt = as.numeric(as.vector(unlist(lognorm_t)))
e_lognorm_p = sqrt(mean((lp - lt)^2))

final.errors = c(e_lognorm,e_lognorm_p) #Speed and power errors
imp.speed = ((e_lognorm - sqrt(mse))/sqrt(mse))*-100 #% Imp. in Speed

##Plotting shift functions
par(mfrow=c(3,2),mar=c(5,5,2,1))
lims = matrix(nrow=num_regimes,ncol=2)
lims[1:2,1] = rep(-0.1,2)
lims[3:6,2] = rep(0.1,2)
lims[1:2,2] = rep(10,2)
lims[3:4,1] = rep(-3,2)
lims[5:6,1] = rep(-5,2)
titles = c("Regime 1: Low speeds, Easterly winds", "Regime 2: Low speeds: Westerly winds", "Regime 3: Moderate speeds, Easterly winds", "Regime 4: Moderate speeds, Westerly winds","Regime 5: High speeds, Easterly winds","Regime 6: High speeds, Westerly winds" )
for(i in 1:num_regimes){
  plot(seq(0,48,1),ests[i,1]*(plnorm(seq(0,48,1),ests[i,2],ests[i,3])),lwd=2,cex=1.5,pch=19,col="black",xlab="Runlength (Hours)",ylab="Shift (m/s)",cex.lab=1.5,cex.axis=1.5,ylim=lims[i,],main=titles[i],cex.main=1.5)
  smoothingSpline = smooth.spline(seq(0,48,1),ests[i,1]*(plnorm(seq(0,48,1),ests[i,2],ests[i,3])), spar=0.05)
  lines(smoothingSpline)
  box(lwd=4)
}

#Saving important output
write.table(ests,file="Code Used in the Book/Chapter4/temp/par_pars.txt",row.names=F,col.names=F)
