#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(circular)
library(changepoint)
library(fields)
library(Matrix)
setwd("H:/DingBook/Files Sharing/") #modify accordingly
source("Code Used in the Book/Chapter4/funcs/asym_build.R")
source("Code Used in the Book/Chapter4/funcs/SEP_fit.R")
source("Code Used in the Book/Chapter4/funcs/shift_update.R")

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
roll.int=6# amount by which we translate every roll
r_speeds = c(0.00, 4.50, 9.00) #refined speed thresholds
r_dirs = c(45,225) #refined direction thresholds
num_regimes = 6 #number of regimes

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines
##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

load("Code Used in the Book/Chapter4/temp/changepoints.R") #importing changepoints in first 30 days
load("Code Used in the Book/Chapter4/temp/indicesout.R") #importing roll indices for first 30 days
load("Code Used in the Book/Chapter4/temp/trainout.R") 
load("Code Used in the Book/Chapter4/temp/predout.R")
lognorm.param = read.table("Code Used in the Book/Chapter4/temp/par_pars.txt") #calibration fun. parameters
trainout_hist = train.out
predout_hist = pred.out
indicesout_hist = indices.out

#***********************************************************************
#*******************FORECASTING*****************************************
#***********************************************************************
num_test = 12 #number of testing hours
roll = 1 #roll index
end.tr = 719 #end time of Phase I (i.e. start of Phase III)
cps = changepoints 
most.recent.CP = tail(cps[which(cps <= end.tr)],1) #the most recently observed CP
st.tr = min(most.recent.CP,end.tr-1) #at least 2 hours for training an asymmetric model
st.tr = max(st.tr,end.tr-3) #at most 4 hours for training as dictated by PACF plots
pred.out = train.out = indices.out  = params.shift1 =list(length = 1)

repeat{
  
  cat(paste("Currently compiling ROLL", roll,"out of 1339 rolls",sep=" "),"\n")
  start_time = Sys.time()
  
  #Training and Testing for the roll
  train_PP = datas[(( (st.tr-1)*N)+1):(end.tr*N),] #training data
  test_PP = datas[((end.tr*N) + 1):((end.tr*N) + num_test*N),] #testing data
  
  #Mast data
  reff1=dirs[st.tr:(end.tr),1];reff1_speed=speed.masts[st.tr:end.tr,1] #mast 1 data
  reff2=dirs[st.tr:(end.tr),2];reff2_speed=speed.masts[st.tr:end.tr,2] #mast 2 data
  reff3=dirs[st.tr:(end.tr),3];reff3_speed=speed.masts[st.tr:end.tr,3] #mast 3 data
  
  #Spatio-temporal average of mast data
  st.av_v = mean(as.numeric(speed.masts[st.tr:end.tr,1:3]),na.rm=T) #speed average
  st.av_d = mean.circular(circular(as.numeric(dirs[st.tr:end.tr,1:3]),type="angles",units="degrees",rotation="clock"),na.rm=TRUE) #dir average
  if(as.numeric(st.av_d) < 0){st.av_d = as.numeric(st.av_d) + 360}
  st.av_d = as.numeric(st.av_d)
  if(is.na(st.av_v) == T){st.av_v = mean(train_PP$Speed)}
  
  #making sure mast data does not contain any NA's and imputing if necessary
  #wind direction data
  reff1[which(is.na(reff1)==T)] = st.av_d
  reff2[which(is.na(reff2)==T)] = st.av_d
  reff3[which(is.na(reff3)==T)] = st.av_d
  #wind speed data
  reff1_speed[which(is.na(reff1_speed)==T)] = st.av_v
  reff2_speed[which(is.na(reff2_speed)==T)] = st.av_v
  reff3_speed[which(is.na(reff3_speed)==T)] = st.av_v
  
  #Temporal averages of mast data
  av.dir1 = mean.circular(circular(reff1,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir1) < 0){av.dir1 = as.numeric(av.dir1) + 360}
  av.speed1 = mean(reff1_speed)
  av.dir2 = mean.circular(circular(reff2,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir2) < 0){av.dir2 = as.numeric(av.dir2) + 360}
  av.speed2 = mean(reff2_speed)
  av.dir3 = mean.circular(circular(reff3,type="angles",units="degrees",rotation="clock"))
  if (as.numeric(av.dir3) < 0){av.dir3 = as.numeric(av.dir3) + 360}
  av.speed3 = mean(reff3_speed)
  directions = c(as.numeric(av.dir1),as.numeric(av.dir2),as.numeric(av.dir3))
  av.speeds = c(av.speed1,av.speed2,av.speed3)
  
  #Number of time points considered in the training data
  Period = dim(train_PP)[1]/N 
  
  #Grouping turbines according to proximity to masts
  loc_all = matrix( rep( t( coords[,3:2] ) , Period ) , ncol = ncol(coords[,2:3]) , byrow = TRUE )
  loc_t_all = matrix(nrow=dim(train_PP)[1],ncol=2)
  x1 = 0
  for (i in 1:Period){
    x2 = rep(i-1,N)
    x1 = c(x1,x2)
  }
  x1 = x1[-1]
  loc_t_all[,1] = x1
  loc_t_all[,2] = rep(0,dim(train_PP)[1])
  mast_locs = matrix(nrow=num_masts,ncol=2) #mast location data
  mast_locs[1,] = c(coords_mast$lat[1],coords_mast$long[1])
  mast_locs[2,] = c(coords_mast$lat[2],coords_mast$long[2])
  mast_locs[3,] = c(coords_mast$lat[3],coords_mast$long[3])
  areas = vector(length = dim(loc_all)[1])
  mast_dists = rdist.earth(mast_locs[,2:1],loc_all)
  for(qq in 1:dim(loc_all)[1]){
    areas[qq] = which(mast_dists[,qq] == min(mast_dists[,qq]))
  }
  
  #Finding direction partition based on wind direction spatio-temporal average
  ##used only for building asymmetric covariance
  if (st.av_d >= 315){R_theta = "N"}
  if (st.av_d >= 0 & st.av_d < 45){R_theta="N"}
  if(st.av_d >= 45 & st.av_d < 135){R_theta="E"}
  if(st.av_d >= 135 & st.av_d < 225){R_theta="S"}
  if (st.av_d >= 225 & st.av_d < 315){R_theta="W"}
  
  #Building asymmetric covariance matrix
  asym.build.res = asym_build(reff1,reff2,reff3,reff1_speed,reff2_speed,reff3_speed,R_theta,Period,coords,num_masts,areas,directions,av.speeds)
  Cov.mats = asym.build.res[[1]]
  N.vec = asym.build.res[[2]]
  coords.mats = asym.build.res[[3]]
  Vel = asym.build.res[[4]]
  D_list = asym.build.res[[5]]
  zz_mat = asym.build.res[[6]]
  
  #Model fitting
  fit.results = matrix(ncol=10,nrow=num_masts)
  loc_mats = list(length = num_masts)
  loc_t_mats = list(length = num_masts)
  dist_mats = list(length = num_masts)
  tdist_mats = list(length = num_masts)
  z_vecs = list(length = num_masts)
  X_mats = list(length = num_masts)
  opt_sets = list(length = num_masts)
  final.cov.mats = list(length = num_masts)
  mean_est = vector(length = num_masts)
  pred_res = pred_res_sep = list(length = num_masts)
  RMSE_modd = RMSE_modd_sep = vector(length = num_masts)
  train_predictions = test_predictions = list(length = num_masts)
  
  cat("MODEL FITTING & PREDICTION - MAY TAKE FEW MINUTES","\n")  
  for (modd in 1:num_masts){
    z = train_PP$Speed[which(areas==modd)]*3.6 #wind speeds in Km/h
    X.m = matrix(nrow=length(z),ncol=1)
    X.m[,1] = rep(0,length(z)) 
    z = z - mean(z,na.rm=T) 
    coords_modd = coords.mats[[modd]]
    loc_modd = matrix( rep( t( coords_modd[,3:2] ) , Period ) , ncol = ncol(coords_modd[,2:3]) , byrow = TRUE )
    loc_t_modd = matrix(nrow=dim(loc_modd)[1],ncol=2)
    x1 = 0
    for (i in 1:Period){
      x2 = rep(i-1,N.vec[modd])
      x1 = c(x1,x2)
    }
    x1 = x1[-1]
    loc_t_modd[,1] = x1
    loc_t_modd[,2] = rep(0,dim(loc_modd)[1])
    spatdist_modd = (rdist.earth(loc_modd,miles=F))#distance 
    tempdist_modd = rdist(loc_t_modd)
    asym.fit.res = SEP_fit(z,tempdist_modd,spatdist_modd,1,10^-5,300)
    #Recovering optimal parameters and other important values
    phi = asym.fit.res[[1]][1]
    nu = asym.fit.res[[1]][2]
    c = asym.fit.res[[1]][3]
    a = asym.fit.res[[1]][4]
    alpha = asym.fit.res[[1]][5]
    delta = asym.fit.res[[1]][6]
    beta = asym.fit.res[[1]][7]
    lambda = asym.fit.res[[1]][8]
    LLs = as.numeric(as.vector(asym.fit.res[[5]]))
    fit.results[modd,] = c(phi,nu,alpha,a,c,delta,beta,lambda,LLs[1],LLs[2])
    loc_mats[[modd]] = loc_modd
    loc_t_mats[[modd]] = loc_t_modd
    dist_mats[[modd]] = spatdist_modd
    tdist_mats[[modd]] = tempdist_modd
    z_vecs[[modd]] = z
    X_mats[[modd]] = X.m
    final.cov.mats[[modd]] = asym.fit.res[[2]]
    S_sep = asym.fit.res[[3]]
    mean_est[modd] = asym.fit.res[[4]][1,1]
    S = asym.fit.res[[2]]
    
    #prediction
    trainvals = vector(length = Period*N.vec[modd])
    K=S
    M = X.m
    Kinv = solve(K)
    K_sep =  S_sep
    Kinv_sep = solve(K_sep)
    testvals = err = testvals_sep = err_sep = vars = vars_sep = vector(length = num_test*N.vec[modd])
    ones = diag(1,nrow=2,ncol=2)
    
    idd = which(areas==modd)
    idd_first = idd[which(idd <= 200)]
    aaaa = 1
    idd_all = vector(length = N.vec[modd]*num_test)
    for (kkk in 1:num_test){
      idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
      aaaa = aaaa + length(idd_first)
    }
    test_modd = test_PP[idd_all,]
    loc_test_modd = do.call(rbind, replicate(num_test, loc_modd[1:N.vec[modd],], simplify=FALSE))
    loc_t_test = matrix(nrow=dim(loc_test_modd)[1],ncol=2)
    ccc = 1
    for (i in 1:num_test){
      loc_t_test[ccc:(ccc+N.vec[modd]-1),1] =  rep(tail(loc_t_modd,1)[1] + i,N.vec[modd])
      ccc = ccc + N.vec[modd]
    }
    loc_t_test[,2] = rep(0,num_test*N.vec[modd])
    
    for (i in 1:length(testvals)){
      m = 0
      
      test.coords = loc_test_modd[i,]
      test.t = loc_t_test[i,]
      
      t1 = matrix(nrow=1,ncol=2)
      t1[1,] = test.coords
      sdist_vec = as.vector(rdist.earth(t1,loc_modd,miles=F))
      
      t1 = matrix(nrow=1,ncol=2)
      t1[1,] = test.t
      tdist_vec = as.vector(rdist(t1,loc_t_modd))
      
      
      k1 = (1-nu)/(1 + a* (tdist_vec)^(2*alpha))
      k2 = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(beta/2)))
      k2[which(sdist_vec == 0)] = k2[which(sdist_vec == 0)] + (nu/(1-nu))
      k12 = k1 * k2
      
      k2_sep = exp(-(c* sdist_vec)/((1 + a* (tdist_vec)^(2*alpha))^(0/2)))
      k2_sep[which(sdist_vec == 0)] = k2_sep[which(sdist_vec == 0)] + (nu/(1-nu))
      k12_sep = k1 * k2_sep
      
      r = vector(length = length(loc_modd[,1]))
      for (j in 1:length(loc_modd[,1])){
        
        V = Vel[modd,]
        D = D_list[[modd]]
        zz = zz_mat[modd,]
        
        h2 = loc_test_modd[i,2] - loc_modd[j,2] #diff in lat
        h1 = loc_test_modd[i,1] - loc_modd[j,1] #diff in lon
        hh = c(h1,h2)
        tt = loc_t_test[i,1] - loc_t_modd[j,1]
        trans= matrix(hh + tt*V*zz,nrow=1,ncol=2)
        r[j] = (1/sqrt(det(ones + ((tt)^2)*D)))*exp(-((as.numeric(trans %*% solve(ones + ((tt)^2)*D) %*% t(trans)))))
      }
      
      k = phi * ( ((1-lambda)*k12) + (lambda*r))
      k = as.matrix(k,nrow=length(k),ncol=1)
      
      k_sep = phi * k12_sep
      k_sep = as.matrix(k_sep,nrow=length(k_sep),ncol=1)
      
      lam = Kinv %*% k  
      lam2 = Kinv_sep %*% k_sep
      
      obs = as.matrix(z)
      pred = (t(lam) %*% obs) 
      pred_sep = (t(lam2) %*% obs) 
      
      pred = pred + mean(train_PP$Speed[which(areas==modd)]*3.6)
      pred_sep = pred_sep + mean(train_PP$Speed[which(areas==modd)]*3.6)
      
      #Prediction variance
      var.pred = (K[1,1] - t(k)%*%(Kinv%*%k))*0.0771604938 #multiplication to transform to m^2/s^2
      var.pred_sep = (K_sep[1,1] - t(k_sep)%*%(Kinv_sep%*%k_sep))*0.0771604938 #multiplication to transform to m^2/s^2
      
      testvals[i] = pred/3.6
      err[i] = (testvals[i] - test_modd$Speed[i])^2
      vars[i] = var.pred
      
      testvals_sep[i] = pred_sep/3.6
      err_sep[i] = (testvals_sep[i] - test_modd$Speed[i])^2
      vars_sep[i] = var.pred_sep
    }
    test_predictions[[modd]] = testvals
    RMSE_asym = sqrt(mean(err))
    RMSE_sep = sqrt(mean(err_sep))
    RMSE_modd[modd] = RMSE_asym
    RMSE_modd_sep[modd] = RMSE_sep
    preds = matrix(nrow=length(testvals),ncol=3)
    preds[,1] = testvals
    preds[,2] = test_modd$Speed
    preds[,3] = vars
    preds_sep = matrix(nrow=length(testvals_sep),ncol=3)
    preds_sep[,1] = testvals_sep
    preds_sep[,2] = test_modd$Speed
    preds_sep[,3] = vars_sep
    
    pred_res[[modd]] = preds
    pred_res_sep[[modd]] = preds_sep
  }
  
  #Persistence
  pred_pers = tail(train_PP$Speed,200)
  pred_pers = rep(pred_pers,times=num_test)
  
  #ASYM & SEP
  for (modd in 1:num_masts){
    idd = which(areas==modd)
    idd_first = idd[which(idd <= 200)]
    aaaa = 1
    idd_all = vector(length = N.vec[modd]*num_test)
    for (kkk in 1:num_test){
      idd_all[aaaa:(aaaa+length(idd_first)-1)] = idd_first + ((kkk-1)*200)
      aaaa = aaaa + length(idd_first)
    }
    if(modd==1){id_1=idd_all}
    if(modd==2){id_2=idd_all}
    if(modd==3){id_3=idd_all}
  }
  fin.res = matrix(nrow=(num_test*200),ncol=3)
  fin.res[id_1,1:3] = pred_res[[1]]
  fin.res[id_2,1:3] = pred_res[[2]]
  fin.res[id_3,1:3] = pred_res[[3]]
  fin.res.sep = matrix(nrow=(num_test*200),ncol=3)
  fin.res.sep[id_1,1:3] = pred_res_sep[[1]]
  fin.res.sep[id_2,1:3] = pred_res_sep[[2]]
  fin.res.sep[id_3,1:3] = pred_res_sep[[3]]
  
  Final.Predictions = matrix(nrow=(num_test*200),ncol=10)
  Final.Predictions[,1] = fin.res[,1] #asym forecasts (mean of posterior)
  Final.Predictions[,2] = pred_pers #per forecasts
  Final.Predictions[,3] = fin.res[,2] #true values
  Final.Predictions[,4] = fin.res.sep[,1] #sep forecasts
  Final.Predictions[,5] = fin.res[,3] #Variance of asym forecasts
  Final.Predictions[,6] = fin.res.sep[,3] #Variance of sep forecasts
  
  
  #####****Calibrating forecasts*****#####
  cat("FORECAST CALIBRATION","\n")
  Vav = mean(train_PP$Speed) #ST-average of wind speed in that roll
  st.ts = end.tr + 1 #start of forecast horizon
  end.ts = st.ts + num_test - 1 #end of forecast horizon
  Dav = mean.circular(circular(dirs[(st.tr:end.tr),1:3],type="angles",units="degrees",rotation="clock"),na.rm=T) #ST-average of wind direction of that roll
  if(Dav < 0){Dav = Dav + 360}
  Dav = as.numeric(Dav)
  #Labeling regimes
  if(Vav < r_speeds[2]){temp1 = 1}
  if(Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1 = 2}
  if(Vav >= r_speeds[3]){temp1 = 3}
  if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
    temp2 = 1}else{temp2=2}
  if(temp1 == 1 & temp2 == 1){Regime = 1}
  if(temp1 == 1 & temp2 == 2){Regime = 2}
  if(temp1== 2 & temp2 == 1){Regime = 3}
  if(temp1 == 2 & temp2 == 2){Regime = 4}
  if(temp1 == 3 & temp2 == 1){Regime = 5}
  if(temp1 == 3 & temp2 == 2){Regime = 6}
  
  theta_h1 = as.numeric(as.vector(lognorm.param[Regime,])) #cal. fn parameters
  #Finding most recently observed regime change
  adj.st.tr = st.tr
  last.cp = tail(cps[which(cps < end.tr)],1) 
  if(length(last.cp) != 0){adj.st.tr = last.cp}
  
  #computing runlength x(t+h)
  rl_t = seq(st.ts-adj.st.tr,(st.ts-adj.st.tr)+num_test-1,1)
  rl = rep(rl_t,each=200)
  
  #computing calibration quantity
  betas1 = theta_h1[1]*plnorm(rl,theta_h1[2],theta_h1[3]) 
  
  #Bounding
  bound = 3
  betas2 = pmin(betas1,bound)
  betas2 = pmax(-bound,betas2)
  fadj2 = fin.res[,1] + betas2
  Final.Predictions[,9] = betas2
  Final.Predictions[,10] = fadj2
  
  Final.Predictions[,7] = betas1 #Record unconstrained amount of shift
  f_adj1 = fin.res[,1] + betas1 
  Final.Predictions[,8] = f_adj1 #Adjusted forecasts
  
  #Recording 
  pred.out[[roll]] = Final.Predictions
  train.out[[roll]] = train_PP
  indices.out[[roll]] = c(st.tr,end.tr)
  trainout_hist[[(length(trainout_hist)+1)]] = train_PP
  predout_hist[[(length(predout_hist)+1)]] = Final.Predictions
  indicesout_hist[[(length(indicesout_hist)+1)]] = c(st.tr,end.tr)
  
  #Finding regime changes in newly revealed data
  newspeeddata = rbind(train_PP,test_PP[1:(N*roll.int),])
  newdirdata = rbind(dirs[st.tr:end.tr,1:3],dirs[(end.tr+1):(end.tr+roll.int),1:3])
  aa = 1;speed.means = dir.means = regspeeds = regdirs = regime.means = vector(length = (Period+roll.int))
  for (i in 1:length(speed.means)){
    speed.means[i] = mean(newspeeddata$Speed[aa:(aa+199)],na.rm=T)
    dir.means[i] = mean.circular(circular(newdirdata[i,1:3],type="angles",units="degrees",rotation="clock"),na.rm=T)
    if(dir.means[i] < 0){dir.means[i] = dir.means[i] + 360}
    if(speed.means[i] < r_speeds[2]){regspeeds[i] = 1}
    if(speed.means[i] >= r_speeds[2] & speed.means[i] < r_speeds[3]){regspeeds[i] = 2}
    if(speed.means[i] >= r_speeds[3]){regspeeds[i] = 3}
    if(dir.means[i] >= r_dirs[1] & dir.means[i] < r_dirs[2]){
      regdirs[i] = 1}else{regdirs[i]=2}
    if(regspeeds[i] == 1 & regdirs[i] == 1){regime.means[i] = 1}
    if(regspeeds[i] == 1 & regdirs[i] == 2){regime.means[i] = 2}
    if(regspeeds[i] == 2 & regdirs[i] == 1){regime.means[i] = 3}
    if(regspeeds[i] == 2 & regdirs[i] == 2){regime.means[i] = 4}
    if(regspeeds[i] == 3 & regdirs[i] == 1){regime.means[i] = 5}
    if(regspeeds[i] == 3 & regdirs[i] == 2){regime.means[i] = 6}
    aa = aa + 200
  }
  diff_reg = vector(length = (length(regime.means) - 1))
  for(i in 2:length(regime.means)){
    diff_reg[i] = regime.means[i] - regime.means[i-1]
  }
  newcps = which(abs(diff_reg) > 0)
  if (length(newcps) != 0){
    tempor = tail(newcps,1)
    newcps = newcps + st.tr - 1
    cps = c(cps,newcps)
  }
  
  #Translating for the new roll
  temppp = end.tr
  end.tr=end.tr+roll.int
  
  #Re-training calibration function using newly revealed observations
  cat("RETRAINING CALIBRATION FUNCTION","\n")
  
  trainout_hist[[1]] = NULL
  predout_hist[[1]] = NULL
  indicesout_hist[[1]] = NULL
  shift.out = train_shift(cps,trainout_hist,indicesout_hist,predout_hist,r_speeds,r_dirs,dirs)
  
  #Updated parameters
  lognorm.param = as.matrix(shift.out[[1]],nrow=num_regimes)
  params.shift1[[roll]] = lognorm.param
  
  if ( (end.tr+12) >= (365*24)){break}
  
  #starting point for next roll
  most_recent_cp = tail(cps[which(cps<end.tr)],1) #most recently observed regime change
  st.tr = max(most_recent_cp,end.tr-3) #at most 4 hours for training according to PACF plots
  st.tr = min(st.tr,end.tr-1) #at least 2 hours for training an asymmetric model
  
  
  roll = roll + 1 #uncomment when using repeat
  end_time = Sys.time()
  
  cat(paste("Run time ",end_time - start_time,sep=" "),"\n")
  
}

#Saving outputs
save(pred.out,file="Code Used in the Book/Chapter4/temp/SEP/predout_test.R")
save(train.out,file="Code Used in the Book/Chapter4/temp/SEP/trainout_test.R")
save(indices.out,file="Code Used in the Book/Chapter4/temp/SEP/indicesout_test.R")
save(cps,file="Code Used in the Book/Chapter4/temp/SEP/changepoints_test.R")
