#***********************************************************************
#**************LIBRARIES AND WORKING DIRECTORY**************************
#***********************************************************************
library(fields)
library(Matrix)
library(circular)
getwd()
setwd("H:/DingBook/Files Sharing/") #modify accordingly

#***********************************************************************
#************PARAMETERS TO BE SPECIFICED BY USER************************
#***********************************************************************
roll.int=6# amount by which we translate every roll
r_speeds = c(0.00, 4.50, 9.00) #refined speed thresholds
r_dirs = c(45,225) #refined direction thresholds
num_regimes = 6 #number of regimes
train_roll = 45 #days

#***********************************************************************
#****************************DATA IMPORT********************************
#***********************************************************************
rawdata <- read.csv("Data Used in the Book/Wind Spatio-Temporal Dataset2.csv",as.is=T,header=T) #import data
training_period = 365 #days
Period = training_period*24 #hours
N = 200 #number of turbines
##restructuring data
dd <- matrix(nrow=(Period*N),ncol=3)
indices = seq(2,400,2)
aa = 1
for(i in 1:Period){
  dd[aa:(aa+N-1),1] = seq(1,N,1)
  dd[aa:(aa+N-1),2] = as.numeric(rawdata[i+4,indices])
  dd[aa:(aa+N-1),3] = as.numeric(rawdata[i+4,indices+1])
  aa = aa + N
}
dd = as.data.frame(dd)
colnames(dd) = c("id","Speed","Power")
datas = dd[1:(training_period*24*N),] #Selecting training dataset

#Coordinates of turbines
coords = matrix(nrow=N,ncol=3)
coords[,1] = seq(1,N,1)
coords[,2] = as.numeric(rawdata[1,2:201])
coords[,3] = as.numeric(rawdata[2,2:201])
coords = as.data.frame(coords)
colnames(coords) = c("id","lat","long")

#Restructuring Mast data
num_masts = 3
coords_mast = matrix(nrow=num_masts,ncol=3)
coords_mast[,1] = seq(1,num_masts,1)
coords_mast[,2] = as.numeric(rawdata[1,202:204])
coords_mast[,3] = as.numeric(rawdata[2,202:204])
coords_mast = as.data.frame(coords_mast)
colnames(coords_mast) = c("id","lat","long")
dirs = speed.masts = matrix(nrow=Period,ncol=num_masts) #direction data from masts
indices = c(402,404,406)
for(i in 1:num_masts){
  speed.masts[,i] = as.numeric(rawdata[ 5:(Period+4),indices[i]])
  dirs[,i] = as.numeric(rawdata[5:(Period+4),indices[i]+1])
}
colnames(dirs) = colnames(speed.masts) = c("m1","m2","m3")

#***********************************************************************
#*******************FORECASTING*****************************************
#***********************************************************************
num_test = 12 #number of hours in forecast horizon
roll = 1 #forecast roll index
end.tr = 719 #end of Phase I data
st.tr = 1 #start of Phase I data

pred.out = train.out = indices.out  = params.out = list(length = 1)

repeat{
  
  cat(paste("Currently compiling ROLL", roll,"out of 1339 rolls",sep=" "),"\n")
  start_time = Sys.time()
  
  train_PP = datas[(( (st.tr-1)*N)+1):(end.tr*N),] #training data
  test_PP = datas[((end.tr*N) + 1):((end.tr*N) + num_test*N),] #testing data
  dirs_PP = dirs[st.tr:end.tr,1:3] #wind direction training data
  
  
  Period = dim(train_PP)[1]/N #number of hours in training data
  
  preds_ar1 = preds_ar2 = preds_ar3 = matrix(ncol=num_test,nrow=N)
  param.ar1  = matrix(ncol=4,nrow=N)
  param.ar2 = matrix(ncol=5,nrow=N)
  
  #computing the wind direction ST-average 
  dir.training = dirs[st.tr:end.tr,1:3] 
  dirav.training = vector(length=dim(dir.training)[1])
  for(jjj in 1:dim(dir.training)[1]){
    dirav.training[jjj] = mean.circular(circular(dir.training[jjj,1:3],type="angles",units="degrees",rotation="clock"),na.rm=T)
    if(dirav.training[jjj] < 0){dirav.training[jjj] = dirav.training[jjj] + 360}
  }
  
  #For each turbine
  for (turb.id in 1:N){
    training = train_PP$Speed[which(train_PP$id == turb.id)] #turbine-specific speeds in training data
    
    #Determining the wind regime based on the most recently observed wind speed and direction
    Vav = (tail(training,1)) #most recent wind speed 
    if (Vav >= r_speeds[1] & Vav < r_speeds[2]){temp1=1}
    if (Vav >= r_speeds[2] & Vav < r_speeds[3]){temp1=2}
    if (Vav >= r_speeds[3]){temp1=3}
    Dav = tail(dirav.training,1) #most recent wind direction 
    if(Dav >= r_dirs[1] & Dav < r_dirs[2]){
      temp2 = 1}else{temp2=2}
    if(temp1 == 1 & temp2 == 1){s_t = 1}
    if(temp1 == 1 & temp2 == 2){s_t = 2}
    if(temp1== 2 & temp2 == 1){s_t= 3}
    if(temp1 == 2 & temp2 == 2){s_t = 4}
    if(temp1 == 3 & temp2 == 1){s_t = 5}
    if(temp1 == 3 & temp2 == 2){s_t = 6}
    Regime = s_t
    
    #Only pick training data that falls in the same regime
    if(Regime==1 || Regime == 2){reg.ids=which(training>=r_speeds[1] & training<r_speeds[2])}
    if(Regime==3 || Regime == 4){reg.ids=which(training>=r_speeds[2] & training<r_speeds[3])}
    if(Regime==5 || Regime == 6){reg.ids=which(training>=r_speeds[3])}
    regdir.ids = which(dirav.training >= r_dirs[1] & dirav.training < r_dirs[2])
    regdir2.ids = seq(1,length(dirav.training),1)[-regdir.ids]
    if(Regime == 1 || Regime == 3 || Regime == 5){reg.ids = intersect(reg.ids,regdir.ids)}
    if(Regime == 2 || Regime == 4 || Regime == 6){reg.ids = intersect(reg.ids,regdir2.ids)}
    train.reg = training[reg.ids]
    
    if(length(train.reg) > 1 & length(unique(train.reg)) > 1 ){ #to ensure enough data for model fitting
    #AR(1) model
    arfit1 <- try(u <- arima(train.reg,order=c(1,0,0),method="ML"), silent = TRUE)
    ERR <- ifelse(class(arfit1) == "try-error", TRUE, FALSE)
    if (ERR == TRUE){
      arfit1 = arima(train.reg,order=c(1,0,0),method="CSS")
    }
    #AR(2) model
    arfit2 <- try(u <- arima(train.reg,order=c(2,0,0),method="ML"), silent = TRUE)
    ERR <- ifelse(class(arfit2) == "try-error", TRUE, FALSE)
    if (ERR == TRUE){
      arfit2 = arima(train.reg,order=c(2,0,0),method="CSS")
    }
    preds_ar1[turb.id,] = as.numeric(predict(arfit1,num_test)$pred)
    preds_ar2[turb.id,] = as.numeric(predict(arfit2,num_test)$pred)
    
    param.ar1[turb.id,1:4] = c(arfit1$coef[1:2], arfit1$sigma2,mean(train.reg))
    param.ar2[turb.id,1:5] = c(arfit2$coef[1:3], arfit2$sigma2,mean(train.reg))
    
    }else{
      preds_ar1[turb.id,] = as.numeric(tail(train.reg,1))
      preds_ar2[turb.id,] = as.numeric(tail(train.reg,1))
      if(turb.id == 1){check = check + 1}
    }
  }
  
  
  
  pred_ar1 = cbind(preds_ar1[,1:num_test])
  pred_ar2 = cbind(preds_ar2[,1:num_test])

  testvals1 = as.vector(pred_ar1)
  testvals2 = as.vector(pred_ar2)

  #####Saving predictions####
  #Persistence
  pred_pers = tail(train_PP$Speed,N)
  pred_pers = rep(pred_pers,times=num_test)
  aa = 1;err_pers=vector(length=(num_test*200));rmse_pers=vector(length=num_test)
  for (i in 1:num_test){
    err_pers[aa:(aa+N-1)] = (tail(train_PP$Speed,N) - test_PP$Speed[aa:(aa+N-1)])^2
    rmse_pers[i] = sqrt(mean(err_pers[aa:(aa+N)]))
    aa = aa + N
  }
  fin.err_pers = sqrt(mean(err_pers))
  
  #TS models
  testingdata = test_PP$Speed
  fin.err = sqrt(mean((testvals1 - testingdata)^2))
  fin.err2 = sqrt(mean((testvals2 - testingdata)^2))

  Final.Predictions = matrix(nrow=(num_test*200),ncol=4)
  Final.Predictions[,1] = testvals1
  Final.Predictions[,2] = pred_pers
  Final.Predictions[,3] = testingdata
  Final.Predictions[,4] = testvals2
  
  
  #Recording
  pred.out[[roll]] = Final.Predictions
  train.out[[roll]] = train_PP
  indices.out[[roll]] = c(st.tr,end.tr)
  params.out[[roll]] = cbind(param.ar1,param.ar2)
  
  #Updating roll
  end.tr=end.tr+roll.int
  st.tr = max(1,end.tr - train_roll*24) #historical data
  
  
  if ( (end.tr+12) >= (365*24)){break}
  
  #Update
  roll = roll + 1 
  end_time = Sys.time()
  
  cat(paste("Run time ",end_time - start_time,sep=" "),"\n")
  
}

#Saving outputs
save(pred.out,file="Code Used in the Book/Chapter4/temp/RSAR/predout_test.R")
save(indices.out,file="Code Used in the Book/Chapter4/temp/RSAR/indicesout_test.R")
save(params.out,file="Code Used in the Book/Chapter4/temp/RSAR/paramsout_test.R")
