####*******Function for implementing circular changepoint detection***********####
##Takes as input:
##x: vector of wind direction observations

##returns as output:
##overall_id: indices of changepoints
################################################################################

CPDD <- function(x){ #input is direction data
  
  x = circular(x,type="angles",units="degrees",rotation="clock") #transforming it to circular format
  pp = length(x)
  #Specifying parameters and initilizations
  x_all = x
  sliding = 1 #parameter for rolling binary segmentation
  begin = 1 #begin index of the roll
  sliding_interval = 24 * 7 #sliding window to run CPD on
  id_overall = list() #list to store all change points 
  epsilon=10 #parameter to use in case no change point is found in a window
  par(mfrow=c(3,1),mar=c(2,2,2,2)) #for plotting
  
  #Change detection
  repeat{
    ending_point = min((begin+sliding_interval-1),pp) #end point of roll
    range = begin: ending_point #range to be considered in the roll
    x = x_all[range] #values for which to run the CPD on
    dir_subset=x
    indices = range #the indices of values to run the CPD on
    kappa.hat = mle.vonmises(x)$kappa #MLE estimate for concentration parameter
    rmax = vector(length = 1000)
    for (i in 1:1000){ #determining distribution of cut off to get the 5 or 1 % cut-off
      set.seed(i)
      simulated_data = rvonmises(n=length(x),mu=circular(0),kappa=kappa.hat)
      rmax[i] = change.point(simulated_data)$rmax
      #cat(i," ")
    }
    cut_level = 0.95 #cut level above which a change point is considered significant
    cut_val = quantile(rmax,cut_level)
    
    #Binary Segmentation code
    x.id = seq(1,length(x),1)
    change = function(s,e){ #function for change point detection that renders the rmax value and the index of candidate change point
      sub = x[s:e]
      temppp = mean.circular(x[s:e],na.rm=T)
      if (temppp < 0){temppp = temppp + 360}
      sub[which(is.na(sub) == T)] = temppp
      detection = change.point(sub)
      index = s + detection$k.r - 1
      out = c(detection$rmax,index)
      return(out)
    }
    
    id = list()
    rlist = list()
    iter = 1
    start = 1
    end = length(x)
    num_partitions = 1
    limits = c(start,end)
    repeat{
      part = 1
      vec = vector(length = 1)
      vec2 = vector(length = 1)
      sig = 0
      for (part in 1:num_partitions){ 
        start = limits[part] + ifelse(part > 1,1,0)
        end = limits[part + 1]
        
        if( (end - start + 1) >= 4){ #minimal sample size to ensure reliable estimation
        
        cp = change(start,end)
        r = cp[1]
        if (r >= cut_val){ #significant change point
          sig = sig + 1
          vec[sig] = cp[2]
          vec2[sig] = r
        }}}
      
      if(sig == 0){ #didn't find any change in any partition
        break
      }
      
      id[[iter]] = vec
      rlist[[iter]] = vec2
      num_partitions = length(unlist(id)) + 1
      limits = c(1,sort(unlist(id)),length(x))
      iter = iter + 1
      if(abs(tail(limits,1) - tail(limits,2)[1]) <= 2){break}
    }
    
    ids = unlist(id)
    if (length(ids) != 0){
      ids = begin + ids - 1}
    if (length(ids) == 0){ids = 0}
    id_overall[[sliding]] = ids
    
    plot(as.numeric(dir_subset),ylim=c(0,360),ylab="Theta",xlab="Time(hrs)",lwd=2)
    0 %in% ids
    if ((0 %in% ids == FALSE)){
      points((ids-begin+1),as.numeric(dir_subset[ids-begin+1]),col="red",pch=15,lwd=2,cex=1.5)
      abline(v=ids-begin+1,lty=2,col="red",lwd=2)
    }
    if ((0 %in% ids == FALSE)){
      begin = max(ids) + 1}
    if((0 %in% ids == TRUE)){begin = begin+sliding_interval-epsilon}
    
    number_cps = length(unlist(id_overall))
    cat("number"," ",number_cps,"\n","Interval",sliding,"\n","IDs",sort(ids),"\n")
    
    cat(sliding,"\n")
    sliding = sliding + 1
    if(ending_point == pp){break}
  }
  par(mfrow=c(1,1))
  
  return(id_overall)
}