####*******Function for MLE (model fitting)****#####
##Takes as input:
##y: vector of wind speeds
##tdist: matrix of temporal distances
##sdist: matrix of spatial distances
##smax: maximum step size for MLE
##stol: tolerance for MLE
##ilim: iteration limit for MLE

##returns as output:
##opt_pars: MLEs for the parameters
################################################################################

SEP_fit <- function(y,tdist,sdist,smax,stol,ilim){
  
  mle1=function(pars){
    pars[1:8] = exp(pars[1:8])
    phi = 1.25*var(y) *(pars[1]/(1 + pars[1]))
    nu = pars[2]/(1+pars[2])
    c = pars[3]
    a = pars[4]
    alpha = pars[5]/(1+pars[5])
    delta = pars[6]
    lambda = beta = 0
    r_t = (1-nu)/(1 + a* (tdist)^(2*alpha))
    r_s = exp(-(c* sdist)/((1 + a* (tdist)^(2*alpha))^(beta/2)))
    mat = which(sdist == 0,arr.ind = T)
    for (i in 1:dim(mat)[1]){
      r_s[mat[i,1],mat[i,2]] = r_s[mat[i,1],mat[i,2]] + (nu / (1- nu))
    }
    
    S_NS = r_t * r_s
    
    S = phi * ( ((1-lambda)*S_NS) + (lambda*Cov.mats[[modd]]))
    
    
    diag(S) = diag(S)+delta+10^-5
    
    mu = 0
    
    check = 0
    temp3 <- try(u <- chol(S), silent = TRUE)
    
    cholError <- ifelse(class(temp3) == "try-error", TRUE, FALSE)
    if (cholError == TRUE){
      S = nearPD(S)$mat
      diag(S) = diag(S)  + 10^-3
      temp3 = chol(S)
      check = 1}
    
    temp4<-2*sum(log(abs(diag(temp3))))
    temp5<-forwardsolve(t(temp3),(y-mu))
    temp6<-t(y - mu)%*%backsolve(temp3,temp5)
    temp<-(temp4+temp6)/2
    return(temp)
  }
  starting = c(log(var(y)),rep(-1,5),2,0)
  optimizing<-nlm(mle1,starting,stepmax=5,steptol=stol,iterlim=ilim,hessian=FALSE,gradtol = 10^-3) 
  
  p1 = 1.25 * var(y) * (exp(optimizing$estimate[1])/(1 + exp(optimizing$estimate[1])))
  p2 = exp(optimizing$estimate[2])/(1 + exp(optimizing$estimate[2]))
  p3 = exp(optimizing$estimate[3])
  p4 = exp(optimizing$estimate[4])
  p5 = exp(optimizing$estimate[5])/(1 + exp(optimizing$estimate[5]))
  p6 = exp(optimizing$estimate[6])
  p7=p8=0
  opt_pars = c(p1,p2,p3,p4,p5,p6,p7,p8)
  r_t = (1-p2)/(1 + p4* (tdist)^(2*p5))
  r_s = exp(-(p3* sdist)/((1 + p4* (tdist)^(2*p5))^(p7/2)))
  mat = which(sdist == 0,arr.ind = T)
  for (i in 1:dim(mat)[1]){
    r_s[mat[i,1],mat[i,2]] = r_s[mat[i,1],mat[i,2]] + (p2 / (1- p2))
  }
  S_NS = r_t * r_s
  
  S = p1 * ( ((1-p8)*S_NS) + (p8*Cov.mats[[modd]]))
  diag(S) = diag(S)+p6 + 10^-5
  
  r_t = (1-p2)/(1 + p4* (tdist)^(2*p5))
  r_s = exp(-(p3* sdist)/((1 + p4* (tdist)^(2*p5))^(0/2)))
  mat = which(sdist == 0,arr.ind = T)
  for (i in 1:dim(mat)[1]){
    r_s[mat[i,1],mat[i,2]] = r_s[mat[i,1],mat[i,2]] + (p2 / (1- p2))
  }
  S_sep = r_t * r_s
  S_sep = p1 * S_sep
  diag(S_sep) = diag(S_sep)+p6 + 10^-5
  
  temp3 <- try(u <- chol(S_sep), silent = TRUE)
  temp4<-2*sum(log(abs(diag(temp3))))
  temp5<-forwardsolve(t(temp3),y)
  temp6<-t(y)%*%backsolve(temp3,temp5)
  temp<-(temp4+temp6)/2
  LL.sep = -temp
  
  mu = as.matrix(rep(0,length(y),ncol=1,nrow=length(y)))
  
  fitting.res = list(length = 5)
  fitting.res[[1]]=opt_pars
  fitting.res[[2]]=S
  fitting.res[[3]]=S_sep
  fitting.res[[4]]=mu
  fitting.res[[5]] = c(-optimizing$minimum,LL.sep)
  return(fitting.res)
}
