####*******Function for computing asymmetric covariance matrix****#####
##Takes as input:
##x1,x2,x3: scalar - wind direction from 3 masts, respectively
##y1,y2,y3: scalar - wind speed from 3 masts, respectively
##r: string - wind direction partition (E,W,N,S)
##p: scalar - number of hours 
##co: matrix Nx3: including coordinates of wind turbines
##nm: number of masts 
##A: vector p*Nx1: labeling observations according to spatial region
##dm: vector px1: spatial averages of wind direction
##sm: vector px1: spatial averages of wind speed

##returns as output:
##Covmats: list containing covariance matrices for each spatial region
###############################################################################
asym_build <- function(x1,x2,x3,y1,y2,y3,r,p,co,nm,A,dm,sm){
  
  
  ####Computing the D matrix for Spatial region 1
  vec3 = matrix(ncol=2,nrow=p)
  for (ii in 1:length(y3)){
    #Easterly < 90
    if (r == "E" & x1[ii] < 90){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    #Easterly > 90
    if (r == "E" & x1[ii] >= 90){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    
    #Southerly < 180
    if (r == "S" & x1[ii] < 180){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly > 180
    if (r == "S" & x1[ii] >= 180){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Westerly > 275
    if (r == "W" & x1[ii] > 270){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 270)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    
    #Westerly < 270
    if (r == "W" & x1[ii] <= 270){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Northerly < 360
    if (r == "N" & x1[ii] < 360){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 275)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    #Northerly > 0
    if (r == "N" & x1[ii] >= 0){
      av.dir=x1[ii]
      av.speed=y1[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    vec3[ii,]=c(deltalong,deltalat)*zz
  }
  #entries of the D matrix
  d12_m1=cov(vec3[,1],vec3[,2])
  d1_m1=var(vec3[,1])
  d2_m1=var(vec3[,2])
  
  ####Computing the D matrix for Spatial region 2
  for (ii in 1:length(y3)){
    if (r == "E" & x2[ii] < 90){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    if (r == "E" & x2[ii] >= 90){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly < 180
    if (r == "S" & x2[ii] < 180){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly > 180
    if (r == "S" & x2[ii] >= 180){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Westerly > 275
    if (r == "W" & x2[ii] > 270){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 270)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    
    #Westerly < 270
    if (r == "W" & x2[ii] <= 270){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Northerly < 360
    if (r == "N" & x2[ii] < 360){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 275)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    #Northerly > 0
    if (r == "N" & x2[ii] >= 0){
      av.dir=x2[ii]
      av.speed=y2[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    
    
    vec3[ii,]=c(deltalong,deltalat)*zz
  }
  #the entries of the D matrix
  d12_m2=cov(vec3[,1],vec3[,2])
  d1_m2=var(vec3[,1])
  d2_m2=var(vec3[,2])
  
  ####Computing the D matrix for Spatial region 3
  for (ii in 1:length(y3)){
    if (r == "E" & x3[ii] < 90){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    if (r == "E" & x3[ii] >= 90){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly < 180
    if (r == "S" & x3[ii] < 180){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly > 180
    if (r == "S" & x3[ii] >= 180){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Westerly > 270
    if (r == "W" & x3[ii] > 270){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 270)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    
    #Westerly < 270
    if (r == "W" & x3[ii] <= 270){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Northerly < 360
    if (r == "N" & x3[ii] < 360){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 275)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    #Northerly > 0
    if (r == "N" & x3[ii] >= 0){
      av.dir=x3[ii]
      av.speed=y3[ii]
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    vec3[ii,]=c(deltalong,deltalat)*zz
  }
  #entries of the D matrix
  d12_m3=cov(vec3[,1],vec3[,2])
  d1_m3=var(vec3[,1])
  d2_m3=var(vec3[,2])
  
  d_pars = list(length=3)
  d_pars[[1]]=c(d1_m1,d2_m1,d12_m1)
  d_pars[[2]]=c(d1_m2,d2_m2,d12_m2)
  d_pars[[3]]=c(d1_m3,d2_m3,d12_m3)
  
  Covmats = list(length=nm) #list containing cov. matrices for each spatial region
  Vel = matrix(nrow=nm,ncol=2) #matrix containing velocity vector for each spatial region
  D_list = list(length = nm) #list containing the D matrix for each spatial region
  zz_mat = matrix(nrow=nm,ncol=2) #vector expressing the directionality
  coordsmats = list(length=nm) 
  Nvec = vector(length = nm)
  
  for (ii in 1:nm){ #for each spatial region
    
    idd = which(A == ii) 
    nval = length(which(idd <= 200))
    coords_mat = co[which(idd <= 200) ,]
    dir_avg = av.dir = as.numeric(dm[ii])
    av.speed = sm[ii]
    
    #Easterly < 90
    if (r=="E" & dir_avg < 90){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    #Easterly > 90
    if (r == "E" & dir_avg >= 90){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Southerly < 180
    if (r == "S" & dir_avg < 180){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 90)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,1)
    }
    
    #Northerly > 0
    if (r == "N" & dir_avg >= 0){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 0)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(-1,-1)
    }
    
    #Southerly > 180
    if (r == "S" & dir_avg >= 180){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    #Westerly > 270
    if (r == "W" & dir_avg > 270){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 270)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    #Northerly > 270
    if (r == "N" & dir_avg > 270){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 270)
      yy = sin(dir.diff*pi/180)*av.speed*3.6
      xx = cos(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,-1)
    }
    
    #Westerly < 270
    if (r == "W" & dir_avg <= 270){
      Lbar = mean(co[,2])
      dir.diff = as.numeric(av.dir - 180)
      yy = cos(dir.diff*pi/180)*av.speed*3.6
      xx = sin(dir.diff*pi/180)*av.speed*3.6
      yylat = yy/110.574 #approximate conversion
      xxlong = xx/(111.32*cos(Lbar*pi/180)) #approximate conversion
      deltalong = xxlong
      deltalat = yylat
      zz = c(1,1)
    }
    
    
    V = c(deltalong,deltalat)
    Vel[ii,] = V
    
    # D = diag(V,2)
    D = matrix(nrow=2,ncol=2)
    D[1,1]=d_pars[[ii]][1]
    D[2,2]=d_pars[[ii]][2]
    D[1,2]=D[2,1]=d_pars[[ii]][3]
    
    D = 2*D #because Schlater's model V ~ N(mu, D/2)
    
    #D = diag(V,2)
    
    
    ones = diag(1,nrow=2,ncol=2)
    
    D_list[[ii]] = D
    zz_mat[ii,] = zz
    
    #Specifying lag matrices
    lat_diff = matrix(nrow=nval,ncol=nval)
    lat_diff = sapply(seq(1,nval,1),function(i) coords_mat$lat[i]-coords_mat$lat) #diff in lat
    H1 = as.vector(lat_diff)
    
    long_diff = matrix(nrow=nval,ncol=nval)
    long_diff = sapply(seq(1,nval,1),function(i) coords_mat$long[i]-coords_mat$long) #diff in long
    H2 = as.vector(long_diff)
    H = as.matrix(cbind(H2,H1)) #long, lat lags
    
    B_mats = list(length = p)
    B_dets = vector(length = p)
    for (i in 1:p){
      B_mats[[i]] = ones + ( ((i-1)^2)*D )
      B_dets[i] = det(B_mats[[i]])
    }
    
    C = list(length = p)
    for (i in 1:p){
      H_a = sweep(H,2, ((i-1)*V*zz))
      vals = rowSums( (H_a %*% solve(B_mats[[i]])) * H_a)
      C[[i]] = (1/sqrt(B_dets[i])) * matrix(exp(-vals),nrow=nval,ncol=nval)
    }
    C_diags = list(length = length(C))
    
    for (iii in 1:length(C)){C_diags[[iii]] = C[[1]]}
    
    C_a = as.matrix(bdiag(C_diags))
    
    bbb = 1
    for (iii in 1:(p-1)){ 
      aaa = 1
      repeat{
        C_a[aaa:(aaa+nval-1),(aaa+bbb*nval):( (2*nval+nval*(bbb-1))+aaa-1)] = C[[iii+1]]
        aaa = aaa+nval
        if (( (2*nval+nval*(bbb-1))+aaa-1)> (p*nval)){break}
      }
      bbb = bbb + 1
    }
    C_a[1:nval,(nval+1):(2*nval)]==C[[2]]
    
    bbb = 1
    for (iii in 1:(p-1)){  
      aaa = nval+1
      repeat{
        C_a[(aaa+(bbb-1)*nval):(aaa+bbb*nval-1),(aaa-nval):(aaa-1)] = t(C[[iii+1]])
        aaa = aaa+nval
        if ( (aaa+bbb*N-1)>p*nval){break}
      }
      bbb = bbb + 1
    }
    Covmats[[ii]] = C_a
    
    coordsmats[[ii]] = coords_mat
    Nvec[ii] = nval
    
  }
  asym_list = list(length = 6)
  asym_list[[1]] = Covmats
  asym_list[[2]] = Nvec
  asym_list[[3]] =coordsmats
  asym_list[[4]] = Vel
  asym_list[[5]] = D_list
  asym_list[[6]] = zz_mat
  
  return(asym_list)
}