################################################################################
#File Description: 
#Function to compute turbine-specific power curves using the Binning method

##Takes as input:
#bindata: data matrix with the 1st column corresponding turbine IDs, 2nd 
##        column corresponding to wind speeds, and 3rd column corresponding 
##        to wind power values. 
#bw     : bin width
#cutin  : Cut-in Speed
#rated  : Rated Speed

##returns as output:
#vbin   : vector of wind speed values (x-axis for the power curve)
#P.est  : List where each element is a vector of estimated wind power values (y-axis for the power curve) 
################################################################################
bin_est <- function(bindata,bw,cutin,rated,numturbs){
  
  out.list = list(length = 2)
  
  plot(bindata$Speed[1:10000],bindata$Power[1:10000],xlab="Speed",ylab="Power") #for visualization of speed-power data
  nbin = (rated-cutin)/bw #number of bins
  vbin=vector("numeric",nbin)
  for (aaa in 1:nbin){
    vbin[aaa] = cutin + aaa*bw
  }
  vbin = c(cutin,vbin) #average speed in each bin
  
  #removing outlier values of power
  out1 = which(bindata$Power < 0)
  out2 = which(is.na(bindata$Power) == T)
  outs = unique(c(out1,out2))
  bindata = bindata[-outs,]
  
  #Binning for every turbine
  set.seed(123);randsamplot = sample(N,6) #randomly selected turbines for plotting
  P.est = list(length = N)
  par(mfrow=c(3,2),mar=c(5,5,3,3))
  for (i in 1:numturbs){
    training.p = bindata[which(bindata[,1] == i),2:3]
    bin.av = vector(length = length(vbin)-1)
    #computing power bin averages
    for (ii in 1:(length(vbin)-1)){
      bin.av[ii] = mean(training.p$Power[which(training.p$Speed >= vbin[ii] & training.p$Speed < vbin[ii+1])])
    }
    #plotting
    if(i %in% randsamplot){
      plot(training.p$Speed,training.p$Power,col="black",lwd=1,pch="+",type="p",cex.lab=1.5,cex.axis=1.5,cex=1,xlab="Wind Speed (m/s)",ylab="Normalized Power",xlim=c(0,25))
      lines(head(vbin+0.25,20),bin.av,col="red",lwd=2,pch=19,type="p")
      grid(nx=NULL,ny=NULL,lwd=1,lty=2)
    }
    P.est[[i]] = bin.av
  }
  par(mfrow=c(1,1))
 
  out.list[[1]] = c(vbin)
  out.list[[2]] = P.est
  return(out.list)
}