####*******Function for estimating the calibration function****#####
##Takes as input:
##x: indices of regime changes
##tdata: list containing training data of each roll in historical data
##idata: list containing starting and ending indices of each roll in historical data
##pdata: list containing predictions of each roll in historical data
##regpart: partitions of wind speed
##regdirpart: partitions of wind direction
##dirs: wind direction data for 3 masts

##returns as output:
##ests: matrix containing parameters of calibration function for each regime
################################################################################



train_shift <- function(x,tdata,idata,pdata,regpart,regdirpart,directions){
  
  r_speeds = regpart
  reg1.data=reg2.data=reg3.data=reg4.data=reg5.data=reg6.data=matrix(nrow=1,ncol=3)
  rlt1 = rlt2 = rlt3 = rlt4 = rlt5 = rlt6 = vector(length=1)
  num_regimes = 6
  num_test=12

  
  for (rr in 1:length(tdata)){
    V = mean(tdata[[rr]]$Speed,na.rm=T)
    
    etr = idata[[rr]][2]
    st = idata[[rr]][1]
    sts = etr + 1
    
    D = mean.circular(circular(directions[(st:etr),1:3],type="angles",units="degrees",rotation="clock"),na.rm=T) #ST-average of wind direction of that roll
    if(D < 0){D = D + 360}
    D = as.numeric(D)
    
    #Labeling regimes
    if(V < r_speeds[2]){temp1 = 1}
    if(V >= r_speeds[2] & V < r_speeds[3]){temp1 = 2}
    if(V >= r_speeds[3]){temp1 = 3}
    if(D >= r_dirs[1] & D < r_dirs[2]){
      temp2 = 1}else{temp2=2}
    if(temp1 == 1 & temp2 == 1){Regime = 1}
    if(temp1 == 1 & temp2 == 2){Regime = 2}
    if(temp1== 2 & temp2 == 1){Regime = 3}
    if(temp1 == 2 & temp2 == 2){Regime = 4}
    if(temp1 == 3 & temp2 == 1){Regime = 5}
    if(temp1 == 3 & temp2 == 2){Regime = 6}

    
    last.cp = tail(x[which(x < etr)],1)
    if(length(last.cp) != 0){st = last.cp}
    
    rl = seq(sts-st,(sts-st)+num_test-1,1)
    true_vals = pdata[[rr]][,3]
    asym_preds = pdata[[rr]][,1]
    rl_t = rl
    rl = rep(rl,each=200)
    if(Regime==1){reg1.data = as.data.frame(rbind(reg1.data,cbind(true_vals,asym_preds,rl)));rlt1 = c(rlt1,rl_t)}
    if(Regime==2){reg2.data = as.data.frame(rbind(reg2.data,cbind(true_vals,asym_preds,rl)));rlt2 = c(rlt2,rl_t)}
    if(Regime==3){reg3.data = as.data.frame(rbind(reg3.data,cbind(true_vals,asym_preds,rl)));rlt3 = c(rlt3,rl_t)}
    if(Regime==4){reg4.data = as.data.frame(rbind(reg4.data,cbind(true_vals,asym_preds,rl)));rlt4 = c(rlt4,rl_t)}
    if(Regime==5){reg5.data = as.data.frame(rbind(reg5.data,cbind(true_vals,asym_preds,rl)));rlt5 = c(rlt5,rl_t)}
    if(Regime==6){reg6.data = as.data.frame(rbind(reg6.data,cbind(true_vals,asym_preds,rl)));rlt6 = c(rlt6,rl_t)}
    }
  reg1.data = reg1.data[-1,]
  reg2.data = reg2.data[-1,]
  reg3.data = reg3.data[-1,]
  reg4.data = reg4.data[-1,]
  reg5.data = reg5.data[-1,]
  reg6.data = reg6.data[-1,]
  rlt1 = rlt1[-1];rlt2=rlt2[-1];rlt3=rlt3[-1];rlt4=rlt4[-1];rlt5=rlt5[-1];rlt6=rlt6[-1]
  mse = mean(c(((reg1.data[,1] - reg1.data[,2])^2),((reg2.data[,1] - reg2.data[,2])^2),((reg3.data[,1] - reg3.data[,2])^2),((reg4.data[,1] - reg4.data[,2])^2),((reg5.data[,1] - reg5.data[,2])^2),((reg6.data[,1] - reg6.data[,2])^2)))
  regimes.data = list(length = num_regimes)
  regimes.data[[1]] = reg1.data
  regimes.data[[2]] = reg2.data
  regimes.data[[3]] = reg3.data
  regimes.data[[4]] = reg4.data
  regimes.data[[5]] = reg5.data
  regimes.data[[6]] = reg6.data
  
  rlt.data = list(length =num_regimes)
  rlt.data[[1]]=rlt1;rlt.data[[2]]=rlt2;rlt.data[[3]]=rlt3;rlt.data[[4]]=rlt4;rlt.data[[5]]=rlt5;rlt.data[[6]]=rlt6
  #Lognormal cdf
  rmse = vector(length = num_regimes)
  ests = matrix(nrow=num_regimes,ncol=3)
  for (reg in 1:num_regimes){
    obj_fn <- function(inputs){
      theta_h = inputs[1:3]
      theta_h[2:3] = exp(theta_h[2:3])
      runlengths = rlt.data[[reg]]
      asym_preds = regimes.data[[reg]][,2]
      true_vals = regimes.data[[reg]][,1]
      betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
      betas = rep(betas_t,each=200)
      f_adj = asym_preds + betas
      err = mean(((true_vals - f_adj)^2))
      
      return(err)
    }
    LSE = nlm(obj_fn,p=rep(0,3),iterlim = 1000,stepmax = 1,steptol = 10^-10)
    rmse[reg] = sqrt(LSE$minimum)
    ests[reg,] = LSE$estimate
  }
  ests[,2:3] = exp(ests[,2:3])
  lognorm_p = lognorm_t = list(length = num_regimes)
  for (rr in 1:num_regimes){
    theta_h = ests[rr,]
    runlengths = rlt.data[[rr]]
    asym_preds = regimes.data[[rr]][,2]
    true_vals = regimes.data[[rr]][,1]
    betas_t = theta_h[1] * plnorm(runlengths,theta_h[2],theta_h[3])
    betas = rep(betas_t,each=200)
    f_adj = asym_preds + betas
    lognorm_p[[rr]] = f_adj
    lognorm_t[[rr]]= true_vals
  }
  lp = as.numeric(as.vector(unlist(lognorm_p)))
  lt = as.numeric(as.vector(unlist(lognorm_t)))
  shift = list()
  shift[[1]] = ests
  shift[[2]] = sqrt(mean((lp - lt)^2))
  shift[[3]] = sqrt(mse)
  
  return(shift)
}